# -*- coding: utf-8 -*-
from six.moves import range
import numpy as np
from copy import deepcopy

from ..general import bobyqa as bobyqa
from ..models.covmodel import get_model
from ..general.isspacetime import isspacetime

def covmodelwls(covmodel,covparam,lag_cov_s,lag_cov_t,
  lag_cov_v,lag_cov_n,lag_th=None,theta=None,ratio=None):
  '''

  This function calculate the value of objective function of 
  weighted least square for covaraince function fitting 
  proposed by Cressie in 1985

  Input:    
    
  covmodel    m             list of m nested covariance models in each of 
                            which the spatial and temporal components are put 
                            in a list as [covmodelS,covmodelT] 
  covparam    m             list of covariance parameters for m covmodels in 
                            each component the parameters are listed as 
                            [sill,[covparamS1,covparamS2,..],
                            [covparamT1,covparamT2,..]]
  lag_cov_s   nls by nlt    2D np array of the meshgrid of spatial lags. nls 
                            and nlt denote number of spatial and temporal lags
                            respectively                                        
  lag_cov_t   nls by nlt    2D np array of the meshgrid of temporal lags
  
  lag_cov_v   nls by nlt    2D np array of the empirical covariance values 
                            in the S/T lag meshgrid, ref. stcov function
  lag_cov_n   nls by nlt    2D np array of the number of data pair counts 
                            in the S/T lag meshgrid
  lag_th      na by 1       Optional. For anisotropic case, the directions to 
                            be evaluted   
  theta       1 by nd-1     vector of angle values that characterize the anisotropy. 
                            In a two dimensional space, angle is the trigonometric angle
                            between the horizontal axis and the principal axis of the
                            ellipse. In a three dimensional space, spherical coordinates
                            are used, such that angle(1) is the horizontal trigonometric
                            angle and angle(2) is the vertical trigonometric angle for the
                            principal axis of the ellipsoid. All the angles are measured
                            counterclockwise in degrees and are between -pi/2 and pi/2.                           
  ratio       1 by nd-1     1D array of vector that characterize the ratio for the length of the axes
                            for the ellipse (in 2D) or ellipsoid (in 3D). In a two dimensional
                            space, ratio is the length of the principal axis of the ellipse
                            divided by the length of the secondary axis, so that ratio<1. 
                            i.e., a_max=a, a_min=ratio*a. 
                            In a three dimensional space, ratio(1) is the length of the principal
                            axis of the ellipsoid divided by the length of the second axis, 
                            whereas ratio(2) is length of the principal axis of the ellipsoid
                            divided by the length of the third axis, so that ratio(1)<1 and
                            ratio(2)>1                                 

  Output: 
        
  objfv       scalar        the objective function from Cressie's 1985 paper

  '''
  if lag_th is None:
    lag_cov_est, dummy = covmodelest(lag_cov_s,lag_cov_t,covmodel,covparam)              
    ersqr = (lag_cov_v - lag_cov_est) ** 2
    wt = lag_cov_n / (lag_cov_est[0][0] ** 2 + lag_cov_est ** 2)
    wt /= wt.sum()
    objfv = wt * ersqr
    objfv = objfv[np.where(~np.isnan(objfv))].sum()
  else:
    lag_cov_est,_=anisocovmodelest(covmodel,covparam,theta,ratio,
                       lag_th,lag_cov_s,lag_cov_t)
    objfv=0.
    for i in range(lag_th.size):
      ersqr = (lag_cov_v[i] - lag_cov_est[i]) ** 2
      wt = lag_cov_n[i] / (lag_cov_est[i][0][0] ** 2 + lag_cov_est[i] ** 2)
      wt /= wt.sum()
      objfvi = wt * ersqr
      objfvii = objfvi[np.where(~np.isnan(objfvi))].sum()                 
      objfv=objfv+objfvii 
    
  return objfv

def covmodelfit(lag_cov_s,lag_cov_t,lag_cov_v,lag_cov_n,covmodel,covparam0,
  theta0=None,ratio0=None,lag_th=None,lower_bnd=None, upper_bnd=None): 
  '''

  This function automatically fits the S/T covariance models to S/T empiricl 
  covariances 

  Input:    
    
  lag_cov_s   nls by nlt    2D np array of the meshgrid of spatial lags. nls 
                            and nlt denote number of spatial and temporal lags
                            respectively                                        
  lag_cov_t   nls by nlt    2D np array of the meshgrid of temporal lags
  
  lag_cov_v   nls by nlt    2D np array of the empirical covariance values 
                            in the S/T lag meshgrid, ref. stcov function. For 
                            the anisotropic case, lag_cov_v should be a list 
                            containing na elements with nls by nlt covariance 
                            estimates at different theta speicified in lag_th. 
  lag_cov_n   nls by nlt    2D np array of the number of data pair counts 
                            in the S/T lag meshgrid
  covmodel    m             list of m nested covariance models in each of 
                            which the spatial and temporal components are put 
                            in a list as [covmodelS,covmodelT] 
  covparam0   m             list of intial values covariance parameters for 
                            m covmodels in each component the parameters are 
                            listed as [sill,[covparamS1,covparamS2,..],
                            [covparamT1,covparamT2,..]]
  theta0      scalar        Direction of principle axis of geometric anisotropy 
                            (in radian)
  ratio0      scalar        The ratio between the maximum and minimum ranges of 
                            the anisotropic ellipse. ratio=maximum/minimum and
                            therefore it has range of [0,1]
  lag_th      1 by na       the spatial or S/T covariance model at angle theta 
                            to be evaluated                       
  lower_bnd   m             Optional. The lower bound for covaraince parameters 
                            with the same format of covparam0. Default is None.
  lower_bnd   m             Optional. The lower bound for covaraince parameters 
                            with the same format of covparam0. Default is None.                           
  Output: 
        
  covparam    m             Optimal covariance estimation
  theta       scalar        Optimal direction of principle axis in anisotropic case.
                            Only availabe if anisotropy parameters are specified.
                            E.g., theta0. 
  ratio       scalar        Optimal ratio between principle and secondary axis in 
                            anistropic case
  opt_val     scalar        Optimal value of the objective function

  Remark: 
  1) the fitting uses the weighted least square criteria for covaraince 
  function fitting proposed by Cressie in 1985. The optimization algorithm is 
  bobyqa obtained from nlopt package at http://ab-initio.mit.edu/nlopt/
  2) details of theta and ratio can refer to stamps.stats.stcovfit.anisocovmodelest

  '''
  def objwls(param,ns,nt):
    ''' This function is used by nlopt to assess the objective function
    At this stage, the objective is based upon the covmodelwls function
    '''  
    if theta0 is None:
      covparam=par2covpar(param,ns,nt)
      return covmodelwls(covmodel,covparam,lag_cov_s,lag_cov_t,
                    lag_cov_v,lag_cov_n)  
    else:
      # I should add some check about the inputs for anisotropy 
      covparam=par2covpar(param[:-2],ns,nt)
      return covmodelwls(covmodel,covparam,lag_cov_s,lag_cov_t,
                    lag_cov_v,lag_cov_n,lag_th,param[-2],param[-1]) 
      
  
  def par2covpar(param,ns,nt):   
    '''This function transforms data formats from the form used for nlopt to 
       the form in the regular covariance model
       The nlopt input format is an 1-D np array, in which the covariance 
       parameters are listed in the form of
       [s1, r11, r12, t11, t12, s2, r2, t2, .....] 
       where 
       si is sill of nested model i, 
       rki is the kth spatial range parameters of nested spatial model i
       tki is the kth temporal range parameters of nested temporal model i
    ''' 
    covparam=[]
    k=0
    for m in range(len(covmodel)):
      if ns[m]>0 and nt[m]>0:
        covparam.append([param[k],param[k+1:k+ns[m]+1],
                       param[k+ns[m]+1:k+ns[m]+nt[m]+1]])
      elif ns[m]>0 and nt[m]==0:
        covparam.append([param[k],param[k+1:k+ns[m]+1]])
      elif ns[m]==0 and nt[m]>0:
        covparam.append([param[k],param[k+1:k+nt[m]+1]]) 
      else:
        covparam.append([param[k],param[k+1:k+ns[m]+1],
                       param[k+ns[m]+1:k+ns[m]+nt[m]+1]])   
      
      k=k+ns[m]+nt[m]+1        
    return covparam
    
  def covpar2par(covparam):    
    '''This function transforms data formats from regular covariance model to 
       the form used for nlopt. 
       
       Remark: The regular covariance model can refer to covmodeldef function
       '''                 
    fit_params=[]
    nm=len(covparam)
    npars=np.zeros(nm,dtype=np.int)
    npart=np.zeros(nm,dtype=np.int)
    for k in range(nm):
      for i,par in enumerate(covparam[k]):
        try:
          for j in range(len(par)):
            if not (par[j] is None):
              fit_params.append(par[j])
          if i==1:
            if par[0] is not None:
              npars[k]=len(par)
          else:
            if par[0] is not None:
              npart[k]=len(par)
        except:
          fit_params.append(par)  
    fit_params=np.array(fit_params)      
    return fit_params,npars,npart          
  
  pars,ns,nt=covpar2par(covparam0)
  if theta0 is not None:
    pars=np.append(pars,[theta0,ratio0])
    
  args=[ns,nt]
  nm=len(covparam0)
  idsill=[0]
  idnrg=[]
  if nm>1:
    k=0
    for i in range(nm):
      if ns[i]>1:        
        idnrg.append(np.arange(k+2,k+ns[i]+1))
      if nt[i]>1:
        idnrg.append(np.arange(k+ns[i]+2,k+ns[i]+nt[i]+1))
      if i < nm-1:  
        idsill.append(k+ns[i]+nt[i]+1)
        k=k+ns[i]+nt[i]+1
  
  idnrg=[item for sublist in idnrg for item in sublist]
  idnrg=np.array(idnrg)
      
  npars=len(pars)
  if theta0 is None:    
    idrg=np.setdiff1d(np.arange(npars),idsill)
    idrg=np.setdiff1d(idrg,idnrg)
  else:
    idrg=np.setdiff1d(np.arange(npars-2),idsill)
    idaniso=np.arange(npars-2,npars)

  if lower_bnd:
    low_bnd=covpar2par(lower_bnd)
  else:  
    low_bnd=np.empty(npars)
    low_bnd[idsill]=pars[idsill]*0.3    
    low_bnd[idrg]=pars[idrg]*0.3
    if idnrg.size>0:
      low_bnd[idnrg]=pars[idnrg]*0.8
      low_bnd[idnrg[np.where(low_bnd[idnrg]<0.5)]]=0.5
    if theta0 is not None:
      low_bnd[idaniso[0]]=pars[idaniso[0]]-np.pi/4
      low_bnd[idaniso[1]]=pars[idaniso[1]]*0.5
  if upper_bnd: 
    up_bnd=covpar2par(upper_bnd)
  else:
    up_bnd=np.empty(npars)
    up_bnd[idsill]=pars[idsill]*2.
    up_bnd[idrg]=pars[idrg]*5.
    if idnrg.size>0:
      up_bnd[idnrg]=pars[idnrg]*1.2
    if theta0 is not None:
      up_bnd[idaniso[0]]=pars[idaniso[0]]+np.pi/4
      up_bnd[idaniso[1]]=np.min([pars[idaniso[1]]*2,1.])
          
  result, opt_val=bobyqa.bobyqa( objwls, pars, args, low_bnd, up_bnd )
  
  if theta0 is None:
    covparam=par2covpar(result,ns,nt)
    return covparam, opt_val
  else:
    covparam=par2covpar(result[:-2],ns,nt)
    theta=result[-2]
    ratio=result[-1]
    return covparam, theta, ratio, opt_val
      
  
    
    
    #    
    ##===============================================================================
    # def autofitcov(CSTguess,COVparams,Smodels,Tmodels):
    #    CSTguess=CSTguess.reshape((-1,3)).tolist()
    #    fittingmodels=[]
    #    for cst,smodel,tmodel in zip(CSTguess,Smodels,Tmodels):
    #        cst.insert(1,smodel)
    #        cst.insert(3,tmodel)
    #        fittingmodels.append(cst)
    # 
    #    fittingmodels = tuple(fittingmodels)
    #    return fitcovariance(fittingmodels,COVparams)
    #===============================================================================

def covmodeldef(covmodel, covparam):
  ''' covariance model definition
  
  SYNTAX:
    fit_models=covmodeldef(covmodel,covparam)
  
  INPUTS:  
  covmodel    m             list of m nested covariance models in each of 
                            which the spatial and temporal components are put 
                            in a list as [covmodelS,covmodelT] 
  covparam    m             list of covariance parameters for m covmodels in 
                            each component the parameters are listed as 
                            [sill,[covparamS1,covparamS2,..],
                            [covparamT1,covparamT2,..]]
  OUTPUTS:                          
  fit_models  n_model by 5  list in which row for nest model number and 
                            col for cell, spatial model, spatial range,
                            temporal model, temporal range
  
  Remark: the use of fit_models is originally designed for widely-used 
  covariance models that has only two parameters for both spatial and 
  temporal models                           
  '''
  fit_models=[]
  for i,(modelS,modelT) in enumerate(covmodel):
    if modelS=='nugget' or modelS=='nug':
      rangeS=np.NaN
      rangeT=np.NaN
    else:
      rangeS=covparam[i][1][0]
      rangeT=covparam[i][2][0]
    fit_models.append([covparam[i][0],modelS,rangeS,
                                      modelT,rangeT])
  return fit_models

def covmodelest(lag_cov_s, lag_cov_t, covmodel, covparam):

  '''
  This function calculate the model predicted value at spatial-temporal coordinate that user input

  Syntax:
  COV_est =covmodelest(lag_cov_s,lag_cov_t,covmodel,covparam)
  
  Input:    
    
  lag_cov_s   nls by nlt    2D np array of the meshgrid of spatial lags. nls 
                            and nlt denote number of spatial and temporal lags
                            respectively
                                        
  lag_cov_t   nls by nlt    2D np array of the meshgrid of temporal lags
  covmodel    m             list of m nested covariance models in each of 
                            which the spatial and temporal components are put 
                            in a list as ['covmodelS/covmodelT']. The details of
                            available covariance model see below
  covparam    m             list of covariance parameters for m covmodels in 
                            each component the parameters are listed as 
                            [sill,[covparamS1,covparamS2,..],
                            [covparamT1,covparamT2,..]]                           
    
  Output:
  COV_est     nls by nlt    2D np array of the model predicted value of 2D meshgrid
  Cov_i       list          list includes m nls by nlt 2D np array for
                            all nested models      
  
  Remark: Five covariance models are available for use now, including
  gaussian(gau), exponential, spherical, holecos, and nugget
    
  
  
  '''

  isST, isSTsep, model_res = isspacetime(covmodel)
  if isST:
    if isSTsep:
      modelS, modelT = model_res
      Ki = []
      for model_s, model_t, param_i in zip(modelS, modelT, covparam):
        if len(param_i) == 1:
          if model_s=='nuggetC' or model_s=='nuggetC':
            sill=param_i
            param_s=[None]
            param_t=[None]
          else:
            print 'covparam is not consisent with covmodel'
            raise 
        else:
          sill, param_s, param_t = param_i
        model_s = get_model(model_s)
        model_t = get_model(model_t)                       
        Ki.append( sill * model_s( lag_cov_s, 1., param_s ) * model_t( lag_cov_t, 1., param_t ) )
      return sum(Ki), Ki # K, KK in matlab
    else:
      (modelS,) = model_res
      Ki=[]
      for model_s, param_i in zip(modelS, covparam):
        sill, param_s, s_t_ratio = param_i
        model_s = get_model(model_s)
        Ki.append(
          sill * model_s(lag_cov_s + s_t_ratio * lag_cov_t, 1., param_s))
      return sum(Ki), Ki # K, KK in matlab
  else:
    (modelS,) = model_res
    Ki = []
    for model_s, param_i in zip(modelS, covparam):
      if len(param_i) == 1:
        if model_s == 'nuggetC' :
          sill = param_i
          param_s = [None]
        else:
          print 'covparam is not consisent with covmodel'
          raise 
      sill, param_s = param_i
      model_s = get_model(model_s)
      Ki.append( sill * model_s(lag_cov_s, 1., param_s)) 
    return sum(Ki), Ki # K, KK in matlab

def anisocovmodelest(covmodel,covparam,theta,ratio,lag_th,lag_cov_s,lag_cov_t=None):
  '''
  Evaluate the spatially-aniostrpy covariance with given directional angle of 
  the prinipal axis with maximum range and its ratios to the other axes, and 
  isotropic model
  
  INPUT:
  covmodel    m             list of m nested isotropic covariance models in each of 
                            which the spatial and temporal components are put 
                            in a list as ['covmodelS/covmodelT']. The details of
                            available covariance model see below
  covparam    m             list of isotropic covariance parameters for m covmodels in 
                            each component the parameters are listed as 
                            [sill,[covparamS1,covparamS2,..],
                            [covparamT1,covparamT2,..]]      
  theta       1 by nd-1     vector of angle values that characterize the anisotropy. 
                            In a two dimensional space, angle is the trigonometric angle
                            between the horizontal axis and the principal axis of the
                            ellipse. In a three dimensional space, spherical coordinates
                            are used, such that angle(1) is the horizontal trigonometric
                            angle and angle(2) is the vertical trigonometric angle for the
                            principal axis of the ellipsoid. All the angles are measured
                            counterclockwise in degrees and are between -pi/2 and pi/2.                           
  ratio       1 by nd-1     1D array of vector that characterize the ratio for the length of the axes
                            for the ellipse (in 2D) or ellipsoid (in 3D). In a two dimensional
                            space, ratio is the length of the principal axis of the ellipse
                            divided by the length of the secondary axis, so that ratio<1. 
                            i.e., a_max=a, a_min=ratio*a. 
                            In a three dimensional space, ratio(1) is the length of the principal
                            axis of the ellipsoid divided by the length of the second axis, 
                            whereas ratio(2) is length of the principal axis of the ellipsoid
                            divided by the length of the third axis, so that ratio(1)<1 and
                            ratio(2)>1
  lag_th     1 by na        the spatial or S/T covariance model at angle theta 
                            to be evaluated
  lag_cov_s  nls by nlt     the spatial or S/T covariance model with spatial 
                            distances to be evaluated
  lag_cov_t  nls by nlt     the temporal lags of S/T covariance model to be evaluated

  OUTPUT:

  Csta       (ns by nt) by na  a list with na length contains S/T covariance model  
                               at every angle      
  Cstai      list          a list with na length. Each component has a list 
                           containing the details of each component of nested
                           model
  
  '''  
    
  isST, isSTsep, model_res = isspacetime(covmodel)
  Ka = [None]*lag_th.size
  Kai = [None]*lag_th.size
  for i, phi in enumerate(lag_th):
    if isST:
      if isSTsep:
        modelS, modelT = model_res
        Ki = []
        for model_s, model_t, param_i in zip(modelS, modelT, covparam):
          if len(param_i)==1:
            if model_s == 'nuggetC' or model_t == 'nuggetC':
              sill=param_i
              param_s = [None]
              param_t = [None]
            else:
              print 'covparam is not consisent with covmodel'
              raise 
          else:
            sill, param_s, param_t = param_i
            if param_s.size==0: 
              param_s=[None]
            if param_t.size==0:
              param_t=[None]
          model_s = get_model(model_s)
          model_t = get_model(model_t)
          if param_s[0] is not None:  #param_s[0] is the correlation length
            param_s = (
              param_s * ratio
              / np.sqrt(
                ratio**2 * np.cos(phi-theta)**2
                + np.sin(phi-theta)**2)
              )
          Ki.append(
            sill * model_s(lag_cov_s, 1., param_s)
            * model_t(lag_cov_t, 1., param_t)
            )
          Ka[i] = sum(Ki)
          Kai[i] = Ki  # K, KK in matlab
      else:
        (modelS,) = model_res
        Ki = []
        for model_s, param_i in zip(modelS, covparam):
          sill, param_s, s_t_ratio = param_i
          model_s = get_model(model_s)
          # Assume param_s[0] is the correlation length
          # This may not be true in non-seperable S/T covariance
          if param_s[0] is not None:  
            param_s = param_s*ratio/np.sqrt(ratio**2*np.cos(phi-theta)**2+np.sin(phi-theta)**2)          
          Ki.append( sill * model_s( lag_cov_s + s_t_ratio * lag_cov_t, 1., param_s ) )
        Ka[i] = sum(Ki)
        Kai[i] = Ki # K, KK in matlab
    else:
      (modelS,) = model_res
      Ki = []
      for model_s, param_i in zip(modelS, covparam):
        if len(param_i) == 1:
          if model_s == 'nuggetC' :
            sill = param_i
            param_s = [None]
          else:
            print 'covparam is not consisent with covmodel'
            raise 
        sill, param_s = param_i
        model_s = get_model(model_s)
        if param_s[0] is not None:
          param_s = (
            param_s * ratio
            / np.sqrt(
              ratio**2 * np.cos(phi-theta)**2
              + np.sin(phi-theta)**2)
            )
        Ki.append(sill * model_s(lag_cov_s, 1., param_s))
      Ka[i]=sum(Ki)
      Kai[i]=Ki # K, KK in matlab  
  return Ka, Kai

def covdownscale(bigcovmod, scale, method=None):
    '''
        covmodel:
            [[c,model_s, bs, model_t, bt],
              ...,
             [c,model_s, bs, model_t, bt]]

        return estimated downscale covariance model
    '''

    model_count = len(bigcovmod)
    #parse covmod to get lower and upper bound
    low_bnd = []
    up_bnd = []
    for cov in bigcovmod:
        low_bnd += [0.5*cov[0], 0.5*cov[2], 0.5*cov[4]]
        up_bnd += [1.5*cov[0], 1.5*cov[2], 1.5*cov[4]]
    low_bnd = np.array(low_bnd)
    up_bnd = np.array(up_bnd)
    big_bt = int(up_bnd[2::3].max()) # bt max, e.g. opt function range

    low_bnd[1::3] = up_bnd[1::3] #spatial not change
    up_bnd[2::3] *= scale #temparal change scale
    init_gss = low_bnd + (up_bnd - low_bnd)*0.5

    smallcovmod = deepcopy(bigcovmod)
    big_cov_mod_f = get_cov_mod_func(bigcovmod)
    
    def opt_func(x, big_cov_mod_f, smallcovmod, big_bt):
        for i, cst in enumerate(np.split(x, len(x)/3)):
            smallcovmod[i][0] = cst[0]
            smallcovmod[i][2] = cst[1]
            smallcovmod[i][4] = cst[2]
        small_cov_mod_f = get_cov_mod_func(smallcovmod)

        range_big_bt = range(big_bt)
        big_lags = np.zeros((len(range_big_bt),2))
        big_lags[:,1] = range_big_bt
        big_cov_z = big_cov_mod_f(big_lags)

        range_small_bt = range(scale)
        small_lags = np.zeros((len(range_small_bt), 2))
        small_lags[:,1] = range_small_bt
        multiplier = np.arange(
            scale,0,-1, dtype=np.float64
            ).reshape((scale,1))
        multiplier[1:,:] *=2
        opt_val = 0.0

        for i in range_big_bt:
            small_lags_copy = deepcopy(small_lags)
            small_lags_copy[:,1] += i*scale
            small_cov_z = small_cov_mod_f(small_lags_copy)
            opt_val +=\
                abs(big_cov_z[i][0]\
                    - (small_cov_z * multiplier / scale**2).sum()
                    )
        print x, opt_val
        return opt_val
            
    args = (big_cov_mod_f, smallcovmod, big_bt)

    result, opt_val = bobyqa.bobyqa(
        opt_func, init_gss, args, low_bnd, up_bnd, stop_val=10**-8)

    #scale small covariance
    for cov in smallcovmod:
        cov[4] /= scale


    return smallcovmod, opt_val
  
def get_cov_mod_func(covmodel):
    '''
        get covariance model as a python function with input lag
    '''
    def cov_func(lag):
        '''
            lag: n by 2 2D np array, column0 is for spatial, 1 for time
        '''
        s_lag = lag[:, 0]
        t_lag = lag[:, 1]
        Ki = []
        for cov in covmodel:
            c, fs, bs, ft, bt = cov
            fs = get_model(fs)
            ft = get_model(ft)
            Ki.append(c*fs(s_lag, 1., bs)*ft(t_lag, 1., bt))
        return sum(Ki).reshape((-1, 1))
    return cov_func

def cal_cov_mod(lag, cov_mod):
    ''' calculate covariance value at each lag
    lag: n by 2 np 2d array, [[s_lag, t_lag],...,[s_lag, t_lag]]
    cov_mod: [[c,model_s, bs, model_t, bt], ..., [c,model_s, bs, model_t, bt]]
    '''
    cov_f = get_cov_mod_func(cov_mod)
    cov_z = cov_f(lag)
    return cov_z

def _index_to_fit_func(index):
    dictionary={"gaussian":_Gau,
                "exponential":_Exp,
                "spherical":_Sph,
                "holecos":_HoC,
                "nugget":_Nug,
                "gau":_Gau,
                "exp":_Exp,
                "sph":_Sph,
                "hoc":_HoC,
                "nug":_Nug,}
    return dictionary[index]
  

def _Gau(bandwidth,lag):
    value = np.exp(-3*(lag/bandwidth)**2)
    return value

def _Exp(bandwidth,lag):
    value = np.exp(-3*(lag/bandwidth))
    return value

def _Sph(bandwidth,lag):
    value=lag.copy()
    boollag = lag <= bandwidth
    value[boollag] = 1.0-1.5*(lag[boollag]/bandwidth)\
                    +0.5*(lag[boollag]/bandwidth)**3;
    boollag = lag > bandwidth 
    value[boollag] = 0.0
    return value;

def _HoC(bandwidth,lag):
    value = np.cos(3.1415926*lag/bandwidth)
    return value

def _Nug(bandwidth,lag):
    value=lag.copy()
    boollag = lag == 0
    value[boollag] = 1.
    boollag = lag != 0
    value[boollag] = 0.
    return value;

if __name__ == "__main__":
    covmodel=[['nuggetC/nuggetC'],['exponentialC/exponentialC'],['exponentialC/exponentialC']]
    # covmodel=[['nugget','nugget'],['exponential','exponential'],['exponential','exponential']]
    covparam=[[0.2,[None],[None]],[0.7,[100],[10]],[0.3,[50],[5]]]
    # fit_models = [[0.7,'exponential',100,'exponential',10],
    #               [0.3,'exponential',50,'exponential',5]]
    lag_s = np.linspace(0,100,11)
    lag_t = np.linspace(0,10,6)
    lag_th = np.linspace(-np.pi/2,np.pi/2,10)
    lag_cov_t, lag_cov_s = np.meshgrid( lag_t, lag_s )
    lag_cov_est,cov_est_i = covmodelest( lag_cov_s,lag_cov_t,covmodel, covparam)
    Ka,Kai=anisocovmodelest(covmodel,covparam,np.pi/6,0.8,lag_th,lag_cov_s,lag_cov_t)
    lag_cov_n = np.ones( lag_cov_est.shape )
    lag_cov_v = lag_cov_est + 0.2*(np.random.rand( *lag_cov_est.shape ) - 0.5)
    objv = covmodelwls(covmodel,covparam,lag_cov_s,lag_cov_t,
                  lag_cov_v,lag_cov_n)
    result, opt_val=covmodelfit(lag_cov_s,lag_cov_t,lag_cov_v,lag_cov_n,covmodel,covparam)
    print objv

    try:
        from matplotlib import pyplot as plt
        lag_s_line = np.linspace(0,100,51)
        lag_t_line = np.linspace(0,10,51)
        lag_cov_t_line, lag_cov_s_line = np.meshgrid( lag_t_line, lag_s_line )
        lag_cov_v_line, dummy = covmodelest( lag_cov_s_line,lag_cov_t_line,
                                     covmodel,covparam )

        plt.figure(1)
        plt.subplot(211)
        plt.plot(lag_s, lag_cov_v[:,0], 'bo', lag_s_line, lag_cov_v_line[:,0], 'b--')
        plt.subplot(212)
        plt.plot(lag_t, lag_cov_v[0], 'ro', lag_t_line, lag_cov_v_line[0], 'r--')

        from mpl_toolkits.mplot3d import Axes3D
        fig = plt.figure(2)
        ax3d = Axes3D(fig)
        ax3d.plot_wireframe(lag_cov_s_line, lag_cov_t_line, lag_cov_v_line)
        ax3d.scatter(lag_cov_s, lag_cov_t,lag_cov_v,c='r')
        plt.show()
    except ImportError,e:
        print 'Warning: Import matplotlib fault, cannot draw.'
        print 'Error Message:',e.message



# -*- coding: utf-8 -*-
# @Author: Chieh-Han Lee
# @Date:   2017-02-24 11:21:31
# @Last Modified by:   Chieh-Han Lee
# @Last Modified time: 2017-02-26 22:14:14

import numpy

from ..general import tensor
from scipy.linalg import svd

def HOSVD(A):

    '''
    High Order Singular Value Decomposition

    Input:


    '''

    U = []
    Uh = []
    nModeSingularValues = []

    for mode in range(A.ndim):
        
        u, s, vh = svd( tensor.unfolding(A, mode+1) )
        U.append(u)
        Uh.append(vh)
        nModeSingularValues.append(s)
    
    S = tensor.ttm(A, U, True)

    return U, S, nModeSingularValues

if __name__ == '__main__':

    A = numpy.zeros((3,3,3))

    A[:,:,0] = numpy.asmatrix([[0.9073, 0.8924, 2.1488],
                            [0.7158, -0.4898, 0.3054],
                            [-0.3698, 2.4288, 2.3753]]).transpose()

    A[:,:,1] = numpy.asmatrix([[1.7842, 1.7753, 4.2495],
                            [1.6970, -1.5077, 0.3207],
                            [0.0151, 4.0337, 4.7146]]).transpose()

    A[:,:,2] = numpy.asmatrix([[2.1236, -0.6631, 1.8260],
                            [-0.0740, 1.9103, 2.1335],
                            [1.4429, -1.7495,-0.2716]]).transpose()

    U, S, D = HOSVD(A)


#!/usr/bin/python
#-*- coding: utf-8 -*-

'''
於2011年5月10號建立
Created on 2011/5/10

參考matlab版的stcov而做，想法與實作方法上有些不同

這是用來算時空分析的變異數的函數，輸入與輸出說明如下：
lagCOVv,lagCOVn,lagCOVs,lagCOVt, = stcov(grid_s,grid_t,grid_v,
                       lagS,lagS_range,
                       lagT,lagT_range):
    輸入變數：
    grid_s：np 2維陣列(因為包含x與y)，為所有測站的空間座標值
    grid_t：np 1維陣列，為所有測量的時間座標值
    grid_v：np 2維陣列，以grid_s為列，grid_t為行所對應的測量值
    lagS：np 1維陣列，變異數空間軸的點
    lagS_range：np 1維陣列，對應該點計算時的空間範圍
    lagT：np 1維陣列，變異數時間軸的點
    lagT_range：np 1維陣列，對應該點計算時的時間範圍
    
    輸出變數：
    lagCOVs：lagCOVv相對應點之空間差值 
            [[0,0,0,0,0]
             [1,1,1,1,1]
             [2,2,2,2,2]
                [...]   ]
    lagCOVt：lagCOVv相對應點之時間差值
            [[1,2,3,4,5]
             [1,2,3,4,5]
             [1,2,3,4,5]
                [...]   ]
    lagCOVv：以lagS為列，lagT為行的2維變異數陣列
    lagCOVn：變異數陣列計算時包含的總資料筆數 
    
    舉例：
           以下都必須轉成np array
    grid_s = [ [1.,3.]
               [2.,4.]
               [5.,7.] ]
    grid_t = [ 1. , 4., 6. ]
    grid_v = [ [1.,2.,3.]
               [4.,np.nan,6.]
               [7.,8.,9.]
    
@作者：顧尚真
@author: KSJ
'''
import itertools
from six.moves import range
import numpy as np
import re
try:
    from ..aot.general.diffarray import diffarray as pdist_numba
    from ..aot.general.diffarray import cdist as cdist_numba
except ImportError, e:
    print 'Warning: function in module "diffarray" cannot be a numba aot because:', e
    from ..general.diffarray import diffarray as pdist_numba
    from ..general.diffarray import cdist as cdist_numba
from ..general.diffarray import diffarray_split

from scipy.spatial.distance import cdist as cdist_scipy
from scipy.spatial import cKDTree as KDTree
from ..general.valstvgx import valstv2stg


### For anisotropy development
def stcov(grid_s,grid_t,grid_v,lagS,lagS_range,lagT=None,lagT_range=None, 
  ang=None, angtol=None, DataObj = None):
  '''
  Estimates the space/time cross covariance between variable Zst 
  with measurements at fixed monitoring stations. The monitoring 
  stations for Zst are located at coordinates cMS, and 
  measuring events are at times tME
  
  SYNTAX:
  COVv,COVn,lagSS,lagTT=  
  stcov(grid_s,grid_t,grid_v,lagS,lagS_range,lagT,lagT_range, , ang=None, 
        angtol=None, DataObj = None)
  
  Input:
  	
  grid_s      ns by 2     2D array    cMS
  grid_t      nt by 1     2D array    tME. For spatial case, one should specify 
                                      a specified time or None for the spatial 
                                      observations
  grid_v      ns by nt    2D array    Data Zst
  lagS        nls by 1    1D array    vector with the r lags    
  lagS_range  nls by 1    1D array    vector with the tolerance for the r lags
  lagT        nlt by 1    1D array    vector with the t lags. default is None
                                      for pure spatial case
  lagT_range  nlt by 1    1D array    vector with the tolerance for the t lags
  ang         scalar                  specified direction in radian for directional covariance. 
                                      Default is None for omnidirectional. 
                                      Direction is in radian and zero denotes 
                                      the horizontal axis
  angtol      scalar                  angle tolerance in radian for directional
                                      covariance evaluation. Default is 20 degree
  	
  Output:
  	
  COVv        nls by nlt  2D array    Empirical covariance at LagS by LagT
  COVn        nls by nlt  2D array    Number of pairs for covariance evaluation
  lagSS       nls by nlt  2D array    Meshed lagS        
  lagTT       nls by nlt  2D array    Meshed lagT 
  
  Remark: this function calculates the pdist-based covariance function
  '''

  #if no GUI, give a no use obj
  # if not DataObj:
  #   from nousedataobj import NoUseDataObj
  #   DataObj = NoUseDataObj()       
  # title = DataObj.getProgressText()
  if DataObj:    
    title = DataObj.getProgressText()   
    DataObj.setProgressRange(0,len(lagS))
    DataObj.setCurrentProgress(0, title + "\n- Calculating Covariance ...")

  # Assure the following parameters have proper dimension or format, e.g., 
  # 1D np array 
  if grid_t is None:
    grid_t=np.array([0]).reshape(1,1)
  
  lagS=lagS.ravel()
  lagS_range=lagS_range.ravel()
  if lagT is None:  
    lagT=np.array([0],ndmin=1)
    lagT_range=np.array([0],ndmin=1)
  else:
    lagT=lagT.ravel()
    lagT_range=lagT_range.ravel()  
  
  if len(grid_s.shape)<2:
    grid_s=np.reshape(grid_s,(grid_s.size,1))
  grid_t=np.asarray(grid_t)
  grid_t=np.reshape(grid_t,(grid_t.size,1))
  grid_v=grid_v.reshape((grid_s.shape[0],grid_t.size))
  
  if angtol is None:
    angtol = np.pi*20/180
  
  #find spatial difference between x and y, then remember each index
  #將空間軸與時間軸各轉為每點相差的值，並記住其index (用"_i_"表示)
  if grid_s.shape[0]<8000:
    s_diff_i_left,s_diff_i_right,s_diff_v, angdiff=diffarray(grid_s,aniso=True)
  else:
    s_diff_i_left,s_diff_i_right,s_diff_v, angdiff= \
          diffarray(grid_s,lagS.max()+lagS_range[-1],aniso=True)
  nd=grid_s.shape[1]
  if nd==1:
    s_diff_v=np.abs(s_diff_v).ravel()
  elif nd==2:           
    s_diff_v=np.sqrt(s_diff_v[:,0]**2+s_diff_v[:,1]**2)
  elif nd==3:
    s_diff_v=np.sqrt(s_diff_v[:,0]**2+s_diff_v[:,1]**2+s_diff_v[:,2]**2)
  
  if len(grid_t)<8000:
    t_diff_i_left,t_diff_i_right,t_diff_v,_=diffarray(grid_t)
  else:
    search_rng = lagT.max()+lagT_range[-1]
    if np.issubdtype(grid_t.dtype, np.datetime64):
      search_rng = np.timedelta64(int(round(search_rng)), re.search('\[(.*?)\]',grid_t.dtype.str).group(1))
    t_diff_i_left,t_diff_i_right,t_diff_v,_= diffarray(grid_t, search_rng)
  t_diff_v=np.abs(t_diff_v).ravel()
  
  # extract a portion of s_diff_x and t_diff_x here
  #  to reduce the memory use
  if ang is None:
    idxs=np.where(s_diff_v<=lagS.max()+lagS_range[-1])
  else:
    if ang+angtol>np.pi/2:
      idxs1=np.where(np.logical_or(np.logical_and(np.logical_and(angdiff<=np.pi/2,
                angdiff>ang-angtol),s_diff_v<=lagS.max()+lagS_range[-1]),np.isnan(angdiff)))
      idxs2=np.where(np.logical_and(np.logical_and(angdiff>-np.pi/2,
                angdiff<ang+angtol-np.pi),s_diff_v<=lagS.max()+lagS_range[-1]))  
      idxs=np.union1d(idxs1[0],idxs2[0]) 
    elif ang-angtol<-np.pi/2:
      idxs1=np.where(np.logical_or(np.logical_and(np.logical_and(angdiff<=ang+angtol,
                angdiff>-np.pi/2),s_diff_v<=lagS.max()+lagS_range[-1]),np.isnan(angdiff)))      
      idxs2=np.where(np.logical_and(np.logical_and(angdiff<=np.pi/2,
                angdiff>ang-angtol+np.pi),s_diff_v<=lagS.max()+lagS_range[-1])) 
      idxs=np.union1d(idxs1[0],idxs2[0])           
    else:
      idxs=np.where(np.logical_or(np.logical_and(np.logical_and(angdiff<=ang+angtol,
                angdiff>ang-angtol),s_diff_v<=lagS.max()+lagS_range[-1]),np.isnan(angdiff)))
    #idxs=np.where(np.logical_and(s_diff_v<=lagS.max()+lagS_range[-1],np.logical_and(angdiff<=ang+angtol,angdiff>ang-angtol))  
  s_diff_i_left=s_diff_i_left[idxs].astype(np.int)
  s_diff_i_right=s_diff_i_right[idxs].astype(np.int)
  s_diff_v=s_diff_v[idxs]
  
  idxt=np.where(t_diff_v<=lagT.max()+lagT_range[-1])
  t_diff_i_left=t_diff_i_left[idxt].astype(np.int)
  t_diff_i_right=t_diff_i_right[idxt].astype(np.int)
  t_diff_v=t_diff_v[idxt]  


  #meshgrid s,t    
  lagTT,lagSS=np.meshgrid(lagT,lagS)

  #create result by lagSS's shape
  lagCOVv = np.empty(lagSS.shape)
  lagCOVv[:]=np.nan

  lagCOVm1 = np.empty(lagSS.shape)
  lagCOVm1[:]=np.nan
  lagCOVm2 = np.empty(lagSS.shape)
  lagCOVm2[:]=np.nan  
  #lagCOVm2 = np.zeros(lagSS.shape)
  lagCOVn = np.zeros(lagSS.shape)
   
  #loop lagS first
  for indexS,m in enumerate(lagS):
    
    #find all index that s_diff_v in range lagS_range
    index_diff_s = np.where(
        (s_diff_v >= m-lagS_range[indexS])
        & (s_diff_v <= m+lagS_range[indexS]) )[0]
    s_diff_i_left_select=s_diff_i_left[index_diff_s]
    if s_diff_i_left_select.size == 0: #no match
      continue
    s_diff_i_right_select=s_diff_i_right[index_diff_s]
    
    # For STAR-GUI
    if DataObj:
      if not DataObj.wasProgressCanceled():
        DataObj.setCurrentProgress(indexS + 1) #rest
      else:
        return False

    #loop lagT then ( if lagS matched )
    for indexT,n in enumerate(lagT):
      index_diff_t=np.where( (t_diff_v >= n-lagT_range[indexT]) & (t_diff_v <= n+lagT_range[indexT]) )[0]
      if DataObj:          
        DataObj.setCurrentProgress(indexS + 1) #rest
                
      t_diff_i_left_select=t_diff_i_left[index_diff_t]
      if t_diff_i_left_select.size == 0:
        continue
      t_diff_i_right_select=t_diff_i_right[index_diff_t]
      #get grid_v that matched range
      grid_v_left_select=grid_v[np.ix_(s_diff_i_left_select,t_diff_i_left_select)]
      grid_v_right_select=grid_v[np.ix_(s_diff_i_right_select,t_diff_i_right_select)]
    	
      #maybe has np.nan, exclude it
      grid_v_not_nan_index= ~(np.isnan(grid_v_left_select) | np.isnan(grid_v_right_select))    
      grid_v_left_select=grid_v_left_select[grid_v_not_nan_index]
      grid_v_right_select=grid_v_right_select[grid_v_not_nan_index]
    	
    	
      lagCOVv[indexS][indexT] = (grid_v_left_select * grid_v_right_select).sum()
      lagCOVm1[indexS][indexT] = grid_v_left_select.sum()
      lagCOVm2[indexS][indexT] = grid_v_right_select.sum()
      lagCOVn[indexS][indexT] = grid_v_left_select.size
        
      if DataObj:          
        DataObj.setCurrentProgress(indexS + 1) #rest
    	
    if DataObj:          
      DataObj.setCurrentProgress(indexS + 1) #rest

                
  lagCOVv/=lagCOVn
  lagCOVm1/=lagCOVn
  lagCOVm2/=lagCOVn
  lagCOVv-=lagCOVm1*lagCOVm2
  
  if DataObj:  
    DataObj.setCurrentProgress(text = title)
  return lagCOVv,lagCOVn,lagSS,lagTT

def stcov_kdtree(
    grid_s, grid_t, grid_v,
    lagS, lagS_range, lagT, lagT_range,
    ang = None, angtol = None, DataObj = None):

    def _assure_dimension(
        grid_s, grid_t, grid_v, lagS, lagS_range, lagT, lagT_range, angtol ): 
        if grid_t is None:
            grid_t=np.array([0]).reshape(1,1)

        lagS = lagS.ravel()
        lagS_range = lagS_range.ravel()
        if lagT is None:  
            lagT=np.array([0],ndmin=1)
            lagT_range=np.array([0],ndmin=1)
        else:
            lagT=lagT.ravel()
            lagT_range=lagT_range.ravel()  

        if len(grid_s.shape)<2:
            grid_s=np.reshape(grid_s,(grid_s.size,1))
        grid_t=np.asarray(grid_t)
        grid_t=np.reshape(grid_t,(grid_t.size,1))
        grid_v=grid_v.reshape((grid_s.shape[0],grid_t.size))
      
        if angtol is None:
            angtol = np.pi*20/180.

        return grid_s, grid_t, grid_v, lagS, lagS_range, lagT, lagT_range, angtol


    grid_s, grid_t, grid_v, lagS, lagS_range, lagT, lagT_range, angtol = \
        _assure_dimension( grid_s, grid_t, grid_v, lagS, lagS_range, lagT, lagT_range, angtol )

    # meshgrid s,t    
    lagTT, lagSS = np.meshgrid( lagT, lagS )

    # create result by lagSS's shape
    index_s_count, index_t_count = lagSS.shape
    lagCOVv = np.zeros((index_s_count, index_t_count))
    lagCOVm1 = np.zeros((index_s_count, index_t_count))
    lagCOVm2 = np.zeros((index_s_count, index_t_count))
    lagCOVn = np.zeros((index_s_count, index_t_count))

    if DataObj:
        title = DataObj.getProgressText()
        total_step_count = index_s_count * index_t_count
        current_step_count = 0
        DataObj.setProgressRange(0, total_step_count)
        sub_title = title + "- Calculating Covariance ...\n"
        DataObj.setCurrentProgress(
            current_step_count, sub_title)

    s_tree = KDTree(grid_s, leafsize=40)
    t_tree = KDTree(grid_t, leafsize=40)

    for indexS, (s_dis, s_tor) in enumerate(zip(lagS, lagS_range)):
        dis_upper = s_dis + s_tor
        set_upper = s_tree.query_pairs(dis_upper)
        dis_lower = s_dis - s_tor - 10**-15
        if dis_lower < 0: # include lag = 0
            dis_lower = 0
            set_lower = set()
            set_upper.update({(i,i) for i in range(grid_s.shape[0])})
        else:
            set_lower = s_tree.query_pairs(dis_lower)
        s_index = np.array(list(set_upper - set_lower))
        if s_index.size == 0: #no match
            continue

        for indexT, (t_dis, t_tor) in enumerate(zip(lagT, lagT_range)):
            # print indexS, indexT
            dis_upper = t_dis + t_tor
            set_upper = t_tree.query_pairs(dis_upper)
            dis_lower = t_dis - t_tor - 10**-15
            if dis_lower < 0: # include lag = 0
                dis_lower = 0
                set_lower = set()
                set_upper.update({(i,i) for i in range(grid_t.shape[0])})
            else:
                set_lower = t_tree.query_pairs(dis_lower)
            t_index = np.array(list(set_upper - set_lower))

            if t_index.size == 0: #no match
                continue

            #get grid_v that matched range
            grid_v_left = grid_v[np.ix_(s_index[:,0], t_index[:,0])]
            grid_v_right = grid_v[np.ix_(s_index[:,1], t_index[:,1])]

            #maybe has np.nan, exclude it
            grid_v_not_nan_index = ~(np.isnan(grid_v_left) | np.isnan(grid_v_right))    
            grid_v_left = grid_v_left[grid_v_not_nan_index]
            grid_v_right = grid_v_right[grid_v_not_nan_index]

            lagCOVv[indexS][indexT] += (grid_v_left * grid_v_right).sum()
            lagCOVm1[indexS][indexT] += grid_v_left.sum()
            lagCOVm2[indexS][indexT] += grid_v_right.sum()
            lagCOVn[indexS][indexT] += grid_v_left.size

            # For STAR-GUI
            if DataObj:
                if not DataObj.wasProgressCanceled():
                    current_step_count += 1
                    DataObj.setCurrentProgress(
                        current_step_count ,
                        sub_title + '({c}/{n})'.format(
                            c=current_step_count,
                            n=total_step_count
                            )
                        ) #rest
                    DataObj.drawGUI()
                else:
                    return False

    lagCOVv/=lagCOVn
    lagCOVm1/=lagCOVn
    lagCOVm2/=lagCOVn
    lagCOVv-=lagCOVm1*lagCOVm2

    return lagCOVv,lagCOVn,lagSS,lagTT

def cov_avg_nd( coord, value, lagC, lagC_range ):
    '''
    get covariance of each range(block) and their pair number
     
    input
    coord: a list of delta matrixs
    value: row x 1, 2d np array
    lagC: a list of lag_limit(1d np array)
    lagC_range: a list of range_limit(1d np array)
    
    return 
    lagCOVv: dim of each lagC's len
    lagCOVn: dim of each lagC's len
    '''
    
    # delta_coord = [] #a list store each delta coordinate
    # for i in range( coord_dim ):
    #     coord_i = coord[ :, i:i+1 ]
    #     delta_coord.append( coord_i - coord_i.T )
        
    delta_value_multi = value * value.T
    delta_value_add_left = value * np.ones( value.T.shape )
    delta_value_add_right = value.T * np.ones( value.shape )
    
    matched_bool_matrix = [] # a coord_dim list store every matched bool matrix ( coord_dim x lag_num )
    for coord_i, lags, ranges in zip( coord, lagC, lagC_range ):
        matched_bool_matrix_i = [] #store 1 of coord_dim matched matrix
        for lag_i, range_i in zip( lags, ranges ):
            matched_bool_matrix_i.append( ( coord_i > lag_i - range_i ) & ( coord_i <= lag_i + range_i ) )
        matched_bool_matrix.append( matched_bool_matrix_i )
    
    #check a range has data included
    # has_data_bool_matrix = []
    # for matched_bool_matrix_i in matched_bool_matrix:
    #     has_data_bool_matrix_i = [] 
    #     for each_matchhed_bool_matrix in matched_bool_matrix_i:
    #         has_data_bool_matrix_i.append( each_matchhed_bool_matrix.any() )
    #     has_data_bool_matrix.append( has_data_bool_matrix_i )
        
    #get index set
    #use itertools to get the index of each possible case(each block)
    lag_dim_len = [ len(i) for i in lagC ]
    index_set = list( itertools.product( *map( range, lag_dim_len ) ) )
    
    #get each block value ( covariance )
    ####set result lagCOV
    lagCOVv = np.empty( lag_dim_len ) #index for each dim
    lagCOVm1 = np.empty( lag_dim_len )
    lagCOVm2 = np.empty( lag_dim_len )
    lagCOVv[:] = np.nan
    lagCOVm1[:] = np.nan
    lagCOVm2[:] = np.nan
    
    lagCOVn = np.zeros( lag_dim_len )
    
    
    for idx in index_set:
        #set the result bool matrix
        res_idx_matrix = np.ones( matched_bool_matrix[0][0].shape, dtype = bool )
        #logical-and for each dim 
        for idx_i, m_b_mat_i in zip( idx, matched_bool_matrix ):
            res_idx_matrix = np.logical_and( res_idx_matrix, m_b_mat_i[ idx_i ] )
        if res_idx_matrix.any() and np.sum( res_idx_matrix ) > 1: #has data and data num at least 2 or cannot calculate covariance
            select_dv = delta_value_multi[ res_idx_matrix ]
            select_dm1 = delta_value_add_left[ res_idx_matrix ]
            select_dm2 = delta_value_add_right[ res_idx_matrix ]
            
            lagCOVv[idx] = select_dv.sum()
            lagCOVm1[idx] = select_dm1.sum()
            lagCOVm2[idx] = select_dm2.sum()
            lagCOVn[idx] =  select_dv.size
        else:
            print "can not calculate"
    # for j in range( lag_num ):    
    #     for i in range( coord_dim ): #row of matched_bool_matrix
    #         has_data_bool_matrix[ i ][ j ] =  matched_bool_matrix[ i ][ j ].any()
           
        
    lagCOVv /= lagCOVn
    lagCOVm1 /= lagCOVn
    lagCOVm2 /= lagCOVn
    lagCOVv -= lagCOVm1*lagCOVm2
    
    return lagCOVv, lagCOVn
        
def stcov_split(grid_s, grid_t, grid_v,
    lagS, lagS_range,
    lagT = None, lagT_range = None, 
    ang = None, angtol = None, DataObj = None):
    '''
        Estimates the space/time cross covariance between variable Zst 
        with measurements at fixed monitoring stations. The monitoring 
        stations for Zst are located at coordinates cMS, and 
        measuring events are at times tME

        split means calculate covariance a piece at a time to prevent memory error

        SYNTAX:
        COVv,COVn,lagSS,lagTT =
        stcov_split(
            grid_s, grid_t, grid_v,
            lagS, lagS_range, lagT, lagT_range,
            ang=None, angtol = None, DataObj = None 
            )

        Input:
          
        grid_s      ns by 2     2D array    cMS
        grid_t      nt by 1     2D array    tME. For spatial case, one should specify 
                                            a specified time or None for the spatial 
                                            observations
        grid_v      ns by nt    2D array    Data Zst
        lagS        nls by 1    1D array    vector with the r lags    
        lagS_range  nls by 1    1D array    vector with the tolerance for the r lags
        lagT        nlt by 1    1D array    vector with the t lags. default is None
                                            for pure spatial case
        lagT_range  nlt by 1    1D array    vector with the tolerance for the t lags
        ang         scalar                  specified direction in radian for directional covariance. 
                                            Default is None for omnidirectional. 
                                            Direction is in radian and zero denotes 
                                            the horizontal axis
        angtol      scalar                  angle tolerance in radian for directional
                                            covariance evaluation. Default is 20 degree
          
        Output:
          
        COVv        nls by nlt  2D array    Empirical covariance at LagS by LagT
        COVn        nls by nlt  2D array    Number of pairs for covariance evaluation
        lagSS       nls by nlt  2D array    Meshed lagS        
        lagTT       nls by nlt  2D array    Meshed lagT 

        Remark: this function calculates the pdist-based covariance function
    '''

    def _assure_dimension(
        grid_s, grid_t, grid_v, lagS, lagS_range, lagT, lagT_range, angtol ): 
        if grid_t is None:
            grid_t=np.array([0]).reshape(1,1)
      
        lagS = lagS.ravel()
        lagS_range = lagS_range.ravel()
        if lagT is None:  
            lagT = np.array([0],ndmin=1)
            lagT_range = np.array([0],ndmin=1)
        else:
            lagT = lagT.ravel()
            lagT_range = lagT_range.ravel()  
      
        if len(grid_s.shape)<2:
            grid_s = np.reshape(grid_s, (grid_s.size,1))
        grid_t = np.asarray(grid_t)
        grid_t = np.reshape(grid_t, (grid_t.size,1))
        grid_v = grid_v.reshape((grid_s.shape[0], grid_t.size))
      
        if angtol is None:
            angtol = np.pi*20/180.

        return grid_s, grid_t, grid_v, lagS, lagS_range, lagT, lagT_range, angtol

    def _diff_space_time(
        grid_s, grid_t, lagS, lagS_range, lagT, lagT_range, ang, angtol ):
        # find spatial difference between x and y, then remember each index
        # 將空間軸與時間軸各轉為每點相差的值，並記住其index (用"_i_"表示)
        # 只取出計算會用到的部份

        has_gui = True if DataObj else False
        has_aniso = False if ang is None else True

        if not has_gui:
            s_diff_i_left, s_diff_i_right, s_diff_v, angdiff= \
                pdist_numba(grid_s, lagS.max() + lagS_range[-1],
                    True, has_aniso)
            t_diff_i_left, t_diff_i_right, t_diff_v,_ = \
            pdist_numba(grid_t, lagT.max() + lagT_range[-1],
                        True, has_aniso)
        else:
            res_s =\
                diffarray_split(grid_s, lagS.max() + lagS_range[-1],
                    True, has_aniso,
                    gui_args=(DataObj,'Covariance\n- Spatial Diffarray...')
                    )
            if res_s:
                s_diff_i_left, s_diff_i_right, s_diff_v, angdiff = res_s
            else:
                return False

            res_t =\
                diffarray_split(grid_t, lagT.max() + lagT_range[-1],
                    True, has_aniso,
                    gui_args=(DataObj,'Covariance\n- Temporal Diffarray...')
                    )
            if res_t:
                t_diff_i_left, t_diff_i_right, t_diff_v, _ = res_t
            else:
                return False

        #各維度平方相加開根號
        s_diff_v **= 2
        s_diff_v = s_diff_v.sum( axis=1 ) # 順便打成一維, keepdims = True )
        s_diff_v **= 0.5
        np.abs( t_diff_v, out = t_diff_v ) # inplace change, faster
        t_diff_v = t_diff_v.ravel()


        # has_gui = True if DataObj else False
        # has_aniso = False if ang is None else True
 
        # if not has_gui:
        #     s_diff_i_left, s_diff_i_right, s_diff_v, angdiff= \
        #         pdist_numba(grid_s, lagS.max() + lagS_range[-1],
        #             True, has_aniso)
        # else:
        #     s_diff_i_left, s_diff_i_right, s_diff_v, angdiff= \
        #         pdist_numba(grid_s, lagS.max() + lagS_range[-1],
        #             True, has_aniso, DataObj)
        #     # for large data with gui,
        #     # here should add gui response for better user experience
        #     s_count = len(grid_s)
        #     s_diff_i_left = np.empty(0, dtype=np.int64)
        #     s_diff_i_right = np.empty(0, dtype=np.int64)
        #     s_diff_v = np.empty((0,grid_s.shape[1]))
        #     angdiff = np.empty(0)
        #     step_length = 4000
        #     step_count = (s_count / step_length) + 1

        #     if DataObj:
        #         title = DataObj.getProgressText()
        #         DataObj.setProgressRange(0, (step_count+1) * step_count / 2)
        #         sub_title = title + "\n- Calculating Covariance ...\nDiffarray..."
        #         DataObj.setCurrentProgress(0, sub_title)
        #         total_step_count = (step_count+1) * step_count / 2
        #         current_step_count = 0

        #     for step_1 in range(0, s_count, step_length):
        #         for step_2 in range(step_1, s_count, step_length):
        #             if step_1 == step_2:
        #                 grid_s_i = grid_s[step_1:step_1+step_length, :]
        #                 d_i_h, d_i_t, d_v, d_a =\
        #                     pdist_numba(
        #                         grid_s_i,
        #                         lagS.max() + lagS_range[-1],
        #                         True, has_aniso)
        #             else:
        #                 grid_s_i = grid_s[step_1:step_1 + step_length, :]
        #                 grid_s_j = grid_s[step_2:step_2 + step_length, :]
        #                 d_i_h, d_i_t, d_v, d_a =\
        #                     cdist_numba(
        #                         grid_s_i, grid_s_j,
        #                         lagS.max() + lagS_range[-1],
        #                         has_aniso)
            
        #             s_diff_i_left = np.append(s_diff_i_left, d_i_h + step_1)
        #             s_diff_i_right = np.append(s_diff_i_right, d_i_t + step_2)
        #             s_diff_v = np.append(s_diff_v, d_v, axis=0)
        #             angdiff = np.append(angdiff, d_a)

        #             if DataObj:
        #                 if not DataObj.wasProgressCanceled():
        #                     current_step_count += 1
        #                     DataObj.setCurrentProgress(
        #                         current_step_count ,
        #                         sub_title + '({c}/{n})'.format(
        #                             c=current_step_count,
        #                             n=total_step_count
        #                             )
        #                         ) #rest
        #                     DataObj.drawGUI()
        #                 else:
        #                     return False

        # #各維度平方相加開根號
        # s_diff_v **= 2
        # s_diff_v = s_diff_v.sum( axis=1 ) # 順便打成一維, keepdims = True )
        # s_diff_v **= 0.5
      
        # t_diff_i_left, t_diff_i_right, t_diff_v,_ = \
        #     pdist_numba(grid_t, lagT.max() + lagT_range[-1],
        #                 True, has_aniso)
        # np.abs( t_diff_v, out = t_diff_v ) # inplace change, faster
        # t_diff_v = t_diff_v.ravel()
      
        # extract a portion of s_diff_x and t_diff_x here
        #  to reduce the memory use
        # 距離跟時間維度已在diffarray裡考慮  這裡只考慮角度
        if ang:
            if ang+angtol>np.pi/2:
                idxs1=np.where(np.logical_or(np.logical_and(np.logical_and(angdiff<=np.pi/2,
                    angdiff>ang-angtol),s_diff_v<=lagS.max()+lagS_range[-1]),np.isnan(angdiff)))
                idxs2=np.where(np.logical_and(np.logical_and(angdiff>-np.pi/2,
                    angdiff<ang+angtol-np.pi),s_diff_v<=lagS.max()+lagS_range[-1]))  
                idxs=np.union1d(idxs1[0],idxs2[0]) 
            elif ang-angtol<-np.pi/2:
                idxs1=np.where(np.logical_or(np.logical_and(np.logical_and(angdiff<=ang+angtol,
                    angdiff>-np.pi/2),s_diff_v<=lagS.max()+lagS_range[-1]),np.isnan(angdiff)))      
                idxs2=np.where(np.logical_and(np.logical_and(angdiff<=np.pi/2,
                    angdiff>ang-angtol+np.pi),s_diff_v<=lagS.max()+lagS_range[-1])) 
                idxs=np.union1d(idxs1[0],idxs2[0])           
            else:
                idxs=np.where(np.logical_or(np.logical_and(np.logical_and(angdiff<=ang+angtol,
                    angdiff>ang-angtol),s_diff_v<=lagS.max()+lagS_range[-1]),np.isnan(angdiff)))

            s_diff_i_left = s_diff_i_left[idxs]
            s_diff_i_right = s_diff_i_right[idxs]
            s_diff_v = s_diff_v[idxs]

            idxt = np.where(t_diff_v<=lagT.max()+lagT_range[-1])
            t_diff_i_left = t_diff_i_left[idxt]
            t_diff_i_right = t_diff_i_right[idxt]
            t_diff_v = t_diff_v[idxt]

        return (s_diff_i_left, s_diff_i_right, s_diff_v,
            t_diff_i_left, t_diff_i_right, t_diff_v)

    def _has_overlay_lag(lag_, range_):
        bd_min = lag_ - range_
        bd_max = lag_ + range_
        for i in range(bd_min.size-1):
            bd_i_min = bd_min[i]
            bd_i_max = bd_max[i]
            bd_j_min = bd_min[i+1:]
            bd_j_max = bd_max[i+1:]
            if ((bd_i_min >= bd_j_max) & (bd_i_max < bd_j_min)).all():
                pass
            else:
                return True
        return False
        

    if DataObj:
        title = DataObj.getProgressText()
    # Assure the following parameters have proper dimension or format, e.g., 
    # 1D np array
    grid_s, grid_t, grid_v, lagS, lagS_range, lagT, lagT_range, angtol = \
        _assure_dimension( grid_s, grid_t, grid_v, lagS, lagS_range, lagT, lagT_range, angtol )

    has_overlay_s = True if _has_overlay_lag(lagS, lagS_range) else False
    has_overlay_t = True if _has_overlay_lag(lagT, lagT_range) else False

    #find spatial difference between x and y, then remember each index
    #將空間軸與時間軸各轉為每點相差的值，並記住其index (用"_i_"表示)
    res =\
        _diff_space_time( grid_s, grid_t, lagS, lagS_range, lagT, lagT_range, ang, angtol )
    if res:
        s_diff_i_left, s_diff_i_right, s_diff_v, t_diff_i_left, t_diff_i_right, t_diff_v = res
    else:
        return False

    # meshgrid s,t    
    lagTT, lagSS = np.meshgrid(lagT, lagS)

    # create result by lagSS's shape
    lagCOVv = np.zeros(lagSS.shape)
    lagCOVm1 = np.zeros(lagSS.shape)
    lagCOVm2 = np.zeros(lagSS.shape)
    lagCOVn = np.zeros(lagSS.shape)

    if DataObj:
        total_step = lagTT.size
        DataObj.setProgressRange(0, total_step)
        sub_title = DataObj.getProgressText() +\
            'Covariance\n- Averaging covariance pairs...'
    #loop lagS first
    for indexS, m in enumerate(lagS):
        #find all index that s_diff_v in range lagS_range
        index_diff_s = np.where(
            (s_diff_v >= m-lagS_range[indexS])
            & (s_diff_v < m+lagS_range[indexS])
            )[0]
        if index_diff_s.size == 0: #no match
            continue
        #loop lagT then ( if lagS matched )
        t_diff_i_left_overlay = np.copy(t_diff_i_left)
        t_diff_i_right_overlay = np.copy(t_diff_i_right)
        t_diff_v_overlay = np.copy(t_diff_v)
        for indexT, n in enumerate(lagT):
            index_diff_t = np.where(
                (t_diff_v_overlay >= n-lagT_range[indexT])
                & (t_diff_v_overlay < n+lagT_range[indexT])
                )[0]
            if index_diff_t.size == 0: #no match
                continue
            #calculate a piece at one time
            index_diff_s_len = index_diff_s.shape[0]
            index_diff_t_len = index_diff_t.shape[0]

            S_STEP = 10000
            T_STEP = 10000

            for s_idx in range(0, index_diff_s_len, S_STEP):
                idx_dif_s_slct = index_diff_s[ s_idx:s_idx + S_STEP ]
                s_diff_i_left_select = s_diff_i_left[ idx_dif_s_slct ]
                s_diff_i_right_select = s_diff_i_right[ idx_dif_s_slct ]
                for t_idx in range(0, index_diff_t_len, T_STEP):
                    idx_dif_t_slct = index_diff_t[ t_idx:t_idx + T_STEP ]
                    t_diff_i_left_select = t_diff_i_left_overlay[ idx_dif_t_slct ]
                    t_diff_i_right_select = t_diff_i_right_overlay[ idx_dif_t_slct ]
                    #get grid_v that matched range
                    grid_v_left_select =\
                        grid_v[np.ix_(s_diff_i_left_select,t_diff_i_left_select)]
                    grid_v_right_select =\
                        grid_v[np.ix_(s_diff_i_right_select,t_diff_i_right_select)]
                    #maybe has np.nan, exclude it
                    grid_v_not_nan_index =\
                        ~(np.isnan(grid_v_left_select) | np.isnan(grid_v_right_select))    
                    grid_v_left_select =\
                        grid_v_left_select[grid_v_not_nan_index]
                    grid_v_right_select =\
                        grid_v_right_select[grid_v_not_nan_index]
                    lagCOVv[indexS][indexT] +=\
                        (grid_v_left_select * grid_v_right_select).sum()
                    lagCOVm1[indexS][indexT] += grid_v_left_select.sum()
                    lagCOVm2[indexS][indexT] += grid_v_right_select.sum()
                    lagCOVn[indexS][indexT] += grid_v_left_select.size

                    # For STAR-GUI
                    if DataObj:
                        if not DataObj.wasProgressCanceled():
                            current_count = indexS * len(lagT) + (indexT + 1)
                            DataObj.setCurrentProgress(
                                current_count,
                                sub_title + '({c}/{n})'.format(
                                    c=current_count,
                                    n=total_step
                                    )
                                ) #rest
                            DataObj.drawGUI()
                        else:
                            return False

            #if not overlay, pop used row (temporal)
            if not has_overlay_t:
                t_diff_i_left_overlay = t_diff_i_left_overlay[~index_diff_t]
                t_diff_i_right_overlay = t_diff_i_right_overlay[~index_diff_t]
                t_diff_v_overlay = t_diff_v_overlay[~index_diff_t]
        #if not overlay, pop used row (spatial)
        if not has_overlay_s:
            s_diff_i_left = s_diff_i_left[~index_diff_s]
            s_diff_i_right = s_diff_i_right[~index_diff_s]
            s_diff_v = s_diff_v[~index_diff_s]
    lagCOVv/=lagCOVn
    lagCOVm1/=lagCOVn
    lagCOVm2/=lagCOVn
    lagCOVv-=lagCOVm1*lagCOVm2
  
    if DataObj: 
        DataObj.setCurrentProgress(text = title)
    return lagCOVv, lagCOVn, lagSS, lagTT

def _stcov(
    grid_s1, grid_t1, grid_v1,
    grid_s2, grid_t2, grid_v2):

    '''
    Calculate spatial temporal covariance
    grid_s nstation by ndim   np 2d array
    grid_t 1 by ntime         np 2d array
    grid_v ns by nt           denote their observations

    return np 2d array
        [[ds, dt, v1, v2],
         [ds, dt, v1, v2],
         [...]]
    '''

    ds = cdist_scipy(grid_s1, grid_s2)
    dt = cdist_scipy(grid_t1.T, grid_t2.T)
    ds_out = np.kron(ds, np.ones(dt.shape))
    dt_out = np.tile(dt, ds.shape)
    v1_out = np.tile(
        grid_v1.T.reshape((-1, 1)),
        (1, grid_t2.shape[1]*grid_s2.shape[0])
        )
    v2_out = np.tile(
        grid_v2.reshape((1, -1)),
        (grid_t1.shape[1]*grid_s1.shape[0], 1)
        )

    stvv = np.dstack((ds_out, dt_out, v1_out, v2_out)).reshape((-1, 4))
    nan_idx = (np.isnan(stvv[:,2]) | np.isnan(stvv[:,3]))
    return stvv[~nan_idx]

def stcov_split2(
    grid_s, grid_t, grid_v,
    lagS, lagS_range,
    lagT = None, lagT_range = None, 
    ang = None, angtol = None, DataObj = None):

    '''
    try to solve memory error problem (not working)
    * grid_t is nt by 1 2d array
    '''

    grid_t = grid_t.T

    lagTT, lagSS = np.meshgrid(lagT, lagS)
    lagCOVv = np.zeros(lagSS.shape)
    lagCOVm1 = np.zeros(lagSS.shape)
    lagCOVm2 = np.zeros(lagSS.shape) 
    lagCOVn = np.zeros(lagSS.shape)

    s_count = grid_s.shape[0]
    t_count = grid_t.size
    s_step_length = 60
    t_step_length = 60
    s_step_count = np.ceil(float(s_count) / s_step_length)
    t_step_count = np.ceil(float(t_count) / t_step_length)
    total_step_count = s_step_count * t_step_count
    total_step_count = (total_step_count+1) * total_step_count/2
    current_step_count = 0
    if DataObj:
        title = DataObj.getProgressText()
        DataObj.setProgressRange(0, total_step_count)
        sub_title = title + "\n- Calculating Covariance..."
        DataObj.setCurrentProgress(0, sub_title)

    st_step = list(
        itertools.product(
            range(0, s_count, s_step_length),
            range(0, t_count, t_step_length)
            )
        )
    for idx, (s_step, t_step) in enumerate(st_step):
        piece_grid_s = grid_s[s_step:s_step+s_step_length, :]
        piece_grid_t = grid_t[:, t_step:t_step+t_step_length]
        piece_grid_v = grid_v[
            s_step:s_step+s_step_length,
            t_step:t_step+t_step_length
            ]
        for s_step2, t_step2 in st_step[idx:]:
            current_step_count += 1
            piece_grid_s2 = grid_s[s_step2:s_step2+s_step_length, :]
            piece_grid_t2 = grid_t[:, t_step2:t_step2+t_step_length]
            piece_grid_v2 = grid_v[
                s_step2:s_step2+s_step_length,
                t_step2:t_step2+t_step_length
                ]
            stvv = _stcov(
                piece_grid_s, piece_grid_t, piece_grid_v,
                piece_grid_s2, piece_grid_t2, piece_grid_v2)
            if stvv.size == 0:
                continue

            for lag_s_idx, (lag_s, lag_s_range) in\
                enumerate(zip(lagS, lagS_range)):

                #ds in range
                select_s_idx = np.where( 
                    (stvv[:, 0] >= lag_s-lag_s_range)
                    & (stvv[:, 0] <= lag_s+lag_s_range)
                    )[0]
                if select_s_idx.size == 0: #no data valid
                    continue
                stvv_select = stvv[select_s_idx]
                for lag_t_idx, (lag_t, lag_t_range) in\
                    enumerate(zip(lagT, lagT_range)):

                    #dt in range (stvv_select)
                    select_t_idx = np.where( 
                        (stvv_select[:, 1] >= lag_t-lag_t_range)
                        & (stvv_select[:, 1] <= lag_t+lag_t_range)
                        )[0]
                    if select_t_idx.size == 0: #no match
                        continue
                    stvv_select_final = stvv_select[select_t_idx]
                    v1_select_final = stvv_select_final[:, 2]
                    v2_select_final = stvv_select_final[:, 3]
                    lagCOVv[lag_s_idx][lag_t_idx] +=\
                        (v1_select_final * v2_select_final).sum()
                    lagCOVm1[lag_s_idx][lag_t_idx] +=\
                        v1_select_final.sum()
                    lagCOVm2[lag_s_idx][lag_t_idx] +=\
                        v2_select_final.sum()
                    lagCOVn[lag_s_idx][lag_t_idx] +=\
                        v1_select_final.size
    lagCOVv /= lagCOVn
    lagCOVm1 /= lagCOVn
    lagCOVm2 /= lagCOVn
    lagCOVv -= lagCOVm1*lagCOVm2

    return lagCOVv, lagCOVn, lagSS, lagTT


if __name__ == "__main__":
  import pandas as pd
  import time
  from ..general.coord2dist import coord2dist
  fname='./examples/Data/dataSEKSGUI_Hard'
  data=pd.read_csv(fname,sep='  ',header=None).values
  Z,cMS,tME,nanloc=valstv2stg(data[:,0:3],data[:,3])
  tME=tME.reshape(tME.size,1)
  stime=time.time()
  s_diff_i_left,s_diff_i_right,s_diff_v,_=diffarray(cMS)
  print 's-time is ' + str(time.time()-stime)
  stime=time.time()
  t_diff_i_left,t_diff_i_right,t_diff_v,_=diffarray(tME)
  print 't-time is ' + str(time.time()-stime)
  
  dist=coord2dist(cMS,cMS)  
  lagS=np.linspace(0,dist.max(),10)
  lagStol=np.ones(lagS.size)*(lagS[1]-lagS[0])
  lagT=np.arange(24)
  lagTtol=np.ones(lagT.size)*0.5
  
  stime=time.time()
  COV,COVn,s,t=stcov_bme(cMS,tME,Z,lagS,lagStol,lagT,lagTtol)
  print 'stcov_bme time is ' + str(time.time()-stime)
  
  stime=time.time()
  COV2,COVn2,s2,t2=stcov(cMS,tME,Z,lagS,lagStol,lagT,lagTtol)
  print 'stcov time is ' + str(time.time()-stime)
  
  stime=time.time()
  COV5,COVn5,s5,t5=stcov_split(cMS,tME,Z,lagS,lagStol,lagT,lagTtol)
  print 'stcov_split time is ' + str(time.time()-stime)
  
  ang=0.
  stime=time.time()
  COV3,COVn3,s3,t3=stcov(cMS,tME,Z,lagS,lagStol,lagT,lagTtol, ang)
  print 'stcov with aniso time is ' + str(time.time()-stime)
  
  stime=time.time()
  COV4,COVn4,s4,t4=stcov(cMS,tME,Z,lagS,lagStol)
  print 'stcov with only spatial time is ' + str(time.time()-stime) 
  
  print 'Is variable "COV" of stcov and stcov_split the same?', np.array_equal( COV2, COV5 )
  print 'Is variable "COVn" of stcov and stcov_split the same?', np.array_equal( COVn2, COVn5 )


import numpy as np
from six.moves import range


def pso(objf, xb, args=(), flag="m", pn=100, it=10, iw=0.3, sc=1.0, si=1.3,
        vmp=None):
    '''
    objf: callable func(x,*args)
          The objective function
    xb: value bound ex. ((10,20),(30,59),...,(12.4,33))
    args: tuple
          Extra arguments passed to objf, i.e. objf(x,*args)
    flag: "M" or "m", (M)aximum or (m)imimum of objf
    pn: int, particle number
    it: int, iteration times
    iw: float, inertia weight
    sc: float, self cognition
    si: float, social interaction
    vmp: max velocity proportion 
         ex. Vmax = boundrange * vmp
         
    return list of optvalue, ex. [15.0,35.0,...,27.5]
    '''
    
    lb,ub = map(np.array,zip(*xb))
    xn=lb.size
    if vmp:
        pass
    else:
        vmp = 0.5 * 1./(pn**(1./xn))
    
    rdarray = np.random.rand(pn,xn)
    pos = rdarray * (ub-lb) + lb
    rdarray = 2 * np.random.rand(pn,xn) - 1
    vm=(ub - lb) * vmp #need vm later
    vel = rdarray * vm
    
    #calculate initial value
    #set self_optimal_fit and position
    slfoptfit = np.zeros(pn)
    slfoptpos = pos #init is the same
    for index_i,pos_i in enumerate(pos):
        slfoptfit[index_i] = objf(pos_i,*args)
        #slfoptpos[i] = pos_i
        
    #set social_optmal_fit and position
    socoptfit = slfoptfit[0]
    socoptpos = slfoptpos[0]
    if flag == "M":
        index = 0
        for pn_i in range(pn):
            if slfoptfit[pn_i] > socoptfit:
                socoptfit = slfoptfit[pn_i]
                index = pn_i
        socoptpos = slfoptpos[index]
    elif flag == "m":
        index = 0
        for pn_i in range(pn):
            if slfoptfit[pn_i] < socoptfit:
                socoptfit = slfoptfit[pn_i]
                index = pn_i
        socoptpos = slfoptpos[index]
        
    #do cycle
    determ = 0
    for it_i in range(it):
        vel = iw * vel + \
              sc * np.random.rand(pn,xn) * (slfoptpos - pos) + \
              si * np.random.rand(pn,xn) * (socoptpos - pos)
        for xn_i in range(vel[0].size):
            vel[:,xn_i] = np.where(abs(vel[:,xn_i]) > vm[xn_i],
                                      vel[:,xn_i]/abs(vel[:,xn_i]+10**-10)*vm[xn_i],
                                      vel[:,xn_i])
#        for vel_i in vel:
#            vel_i = np.where(abs(vel_i) > vm,vel_i/abs(vel_i)*vm,vel_i)
        pos += vel
        for xn_i in range(pos[0].size):
            pos[:,xn_i] = np.where(pos[:,xn_i] > ub[xn_i],
                                      ub[xn_i],
                                      pos[:,xn_i])
            pos[:,xn_i] = np.where(pos[:,xn_i] < lb[xn_i],
                                      lb[xn_i],
                                      pos[:,xn_i])
#        for pos_i in pos:
#            pos_i = np.where(pos_i > ub,ub,pos_i)
#            pos_i = np.where(pos_i < lb,lb,pos_i)
        slfoptfit_temp = np.zeros(pn)
#        slfoptpos_temp = pos
        for index_i,pos_i in enumerate(pos):
            slfoptfit_temp[index_i] = objf(pos_i,*args)
        if flag == "M":
            #compare self_optimal_fit
            if np.any(slfoptfit_temp > slfoptfit):
                optindex = np.where(slfoptfit_temp > slfoptfit)
                slfoptfit[optindex] = slfoptfit_temp[optindex]
                slfoptpos[optindex] = pos[optindex]
            #compare social_optimal_fit
            if np.any(slfoptfit > socoptfit):
                socoptfit = slfoptfit.max()
                socoptpos = pos[slfoptfit == socoptfit][0] 
                #need only one, the first one is choosen
        elif flag == "m":
#            if abs(slfoptfit.min()-socoptfit) <0.0001 and \
#               it_i > 20 and \
#               np.all(abs(pos[slfoptfit == slfoptfit.min()][0]-socoptpos) < 0.0001) :
#                determ += 1
#                if determ == 5:
#                    break
#            else:
#                determ = 0
            #compare self_optimal_fit
            if np.any(slfoptfit_temp < slfoptfit):
                optindex = np.where(slfoptfit_temp < slfoptfit)
                slfoptfit[optindex] = slfoptfit_temp[optindex]
                slfoptpos[optindex] = pos[optindex]
            #compare social_optimal_fit
            if np.any(slfoptfit < socoptfit):
                socoptfit = slfoptfit.min()
                socoptpos = pos[slfoptfit == socoptfit][0] 
                #need only one, the first one is choosen
    #print "iter_times = ",it_i,"objfuncvalue = ",socoptfit
    return socoptpos.tolist(), socoptfit

if __name__ == "__main__":
    def objf(x):
        y=x[1]
        x=x[0]
        return x**2+2*x*y+5+y**2+2*y+3 #min = 4
    xb = [[-90,30],[-2,30]]
    ans=pso(objf,xb,args=(),flag="m",
        pn=98,it=50,iw=0.3,sc=1.0,si=1.3,
        vmp=None)
    print ans
    
##    ans2=scipy.optimize.fminbound(objf,np.array([-90.,30.]),np.array([30.,60.]))
##    print ans2




# -*- coding: utf-8 -*-
# import math
from six.moves import range
import numpy as np
# import scipy.ndimage.filters.gaussian_filter as gaussian_filter
#import logging
#
#log_manager = logging.getLogger('debug_log')
#log_manager.setLevel(logging.DEBUG)
#fh = logging.FileHandler('debug.txt')
#formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
#fh.setFormatter(formatter)
#log_manager.addHandler(fh)
            
def kernelsmoothing(grid_s,grid_t,grid_z,bs,bt,ktype = "gaussian", DataObj = None):
    '''
    grid_s: row by 2 np array
    grid_t: 1 by col np array
    grid_z: row by col np array
    bs: float
    bt: flaot
    ktype: string
    DataObj: for GUI use
    
    return grid_trend, raw by col np array, or return False if fail
    '''
    if not DataObj:
        from ..bme.nousedataobj import NoUseDataObj
        DataObj = NoUseDataObj()
        
    title = DataObj.getProgressText()
        
    #use old code ( grid_t was 1d np array )
    grid_t = grid_t[0]
    
    #determ func
    func = TypeToFunc(ktype)
    
    #create grid_trend
    grid_trend = grid_z.copy()
    
    DataObj.setProgressRange(0,len(grid_s))
    DataObj.setCurrentProgress(0, title + "\n- By Kernel Smoothing...")
    for index_s in range(grid_s.shape[0]):
        if not DataObj.wasProgressCanceled():  
            for index_t in range(grid_t.shape[0]):
                if np.isnan(grid_z[index_s][index_t]):
                    pass
                else:
                    ds = np.sqrt(((grid_s - grid_s[index_s])**2).sum(axis=1))
                    dt = np.abs(grid_t - grid_t[index_t])
                    
#                    index_ss = np.where(ds<=bs)
#                    index_tt = np.where(dt<=bt)
#                    selected_ds = ds[index_ss]
#                    selected_dt = dt[index_tt]
#                    
#                    selected_grid_z = grid_z[np.ix_(index_ss[0],index_tt[0])]
                    
                    ds_nomal = (np.array([ds]).T / bs)**2
                    dt_nomal = (dt / bt)**2
#                    ds_nomal = (np.array([selected_ds]).T / bs)**2
#                    dt_nomal = (selected_dt / bt)**2
                    
                    dr_matrix = ds_nomal + dt_nomal
                    kernel_w = func(dr_matrix)
                    
                    up = kernel_w * grid_z
                    kernel_w[np.isnan(grid_z)] = np.nan
                    down = kernel_w
                    grid_trend[index_s][index_t] = up[~np.isnan(up)].sum() / down[~np.isnan(down)].sum()
            
#                    up = kernel_w * selected_grid_z
#                    kernel_w[np.isnan(selected_grid_z)] = np.nan
#                    down = kernel_w
#                    grid_trend[index_s][index_t] = up[~np.isnan(up)].sum() / down[~np.isnan(down)].sum()
                    
            DataObj.setCurrentProgress(index_s + 1)
        else:
            return False
        
    DataObj.setCurrentProgress(text = title)
    return grid_trend
 
def kernelsmoothing_est(grid_s, grid_t, grid_z, 
                        est_grid_s, est_grid_t,
                        bs, bt, ktype = "gau", DataObj = None):



    if not DataObj:
        from ..bme.nousedataobj import NoUseDataObj
        DataObj = NoUseDataObj()

    title = DataObj.getProgressText()
    
    #determ func
    func = TypeToFunc(ktype)
    
    grid_t = grid_t[0]
    est_grid_t = est_grid_t[0]
    
    #create grid_trend
    est_grid_trend = np.empty((est_grid_s.shape[0],est_grid_t.shape[0]))
    est_grid_trend[:] = np.nan
    
    DataObj.setProgressRange(0,len(est_grid_s))
    DataObj.setCurrentProgress(0, title + "\n- By Kernel Smoothing...")
    for index_s in range(len(est_grid_s)):
        if not DataObj.wasProgressCanceled():  
            for index_t in range(len(est_grid_t)):
                ds = np.sqrt(((grid_s - est_grid_s[index_s])**2).sum(axis=1))
                dt = np.abs(grid_t - est_grid_t[index_t])
                
    #            index_ss = np.where(ds<=bs)
    #            index_tt = np.where(dt<=bt)
    #            selected_ds = ds[index_ss]
    #            selected_dt = dt[index_tt]
    #            selected_grid_z = grid_z[np.ix_(index_ss[0],index_tt[0])]
                
                ds_nomal = (np.array([ds]).T / bs)**2
                dt_nomal = (dt / bt)**2
    #            ds_nomal = (np.array([selected_ds]).T / bs)**2
    #            dt_nomal = (selected_dt / bt)**2
                
                dr_matrix = ds_nomal + dt_nomal
                kernel_w = func(dr_matrix)
                
                up = kernel_w * grid_z
                kernel_w[np.isnan(grid_z)] = np.nan
                down = kernel_w
                est_grid_trend[index_s][index_t] = up[~np.isnan(up)].sum() / down[~np.isnan(down)].sum()
            DataObj.setCurrentProgress(index_s + 1)
        else:
            return False
#            up = kernel_w * selected_grid_z
#            kernel_w[np.isnan(selected_grid_z)] = np.nan
#            down = kernel_w
#            est_grid_trend[index_s][index_t] = up[~np.isnan(up)].sum() / down[~np.isnan(down)].sum()
            
#    print 'grid_s=', grid_s
#    print 'grid_t=', grid_t
#    print 'grid_z=', grid_z
#    print 'grid_s_est=', est_grid_s
#    print 'grid_t_est=', est_grid_t
#    print 'grid_tr_est=', est_grid_trend
    return est_grid_trend

#def kernelsmoothing_cv(grid_s,grid_t,grid_z,bs,bt,ktype = "gaussian",sample = None):
#    #determ func
#    func = TypeToFunc(ktype)
#    
#    #create grid_trend
#    grid_trend = grid_z.copy()
#    
##    #sample list 
##    try:
##        sample_index_list = random.sample([[i,j] for i in range(len(grid_s)) for j in range(len(grid_t)) if ~np.isnan(grid_z[i][j])],sample)
##    except ValueError:
##        sample_index_list = [[i,j] for i in range(len(grid_s)) for j in range(len(grid_t)) if ~np.isnan(grid_z[i][j])]
#
#    
##    #get true sample number
##    samplenumber = len(sample_index_list)
##    
##    for (index_s,index_t) in sample_index_list:
#    for index_s in range(len(grid_s)):
#        for index_t in range(len(grid_t)):
#            if np.isnan(grid_z[index_s][index_t]):
#                pass
#            else:
#                ds = np.sqrt(((grid_s - grid_s[index_s])**2).sum(axis=1))
#                dt = np.abs(grid_t - grid_t[index_t])
#                
#                index_ss = np.where(ds<=bs)
#                index_tt = np.where(dt<=bt)
#                selected_ds = ds[index_ss]
#                selected_dt = dt[index_tt]
#                selected_grid_z = grid_z[np.ix_(index_ss[0],index_tt[0])]
#                          
#                ds_nomal = (np.array([selected_ds]).T / bs)**2
#                dt_nomal = (selected_dt / bt)**2
#                
#                dr_matrix = ds_nomal + dt_nomal
#                kernel_w = func(dr_matrix)
#                
#                up = kernel_w * selected_grid_z
#                kernel_w[np.isnan(selected_grid_z)] = np.nan
#                down = kernel_w
#                
#                #abstract self
#                
#                grid_trend[index_s][index_t] = (up[~np.isnan(up)].sum() - grid_z[index_s][index_t]) / (down[~np.isnan(down)].sum() - 1.)
#    error = (grid_z - grid_trend )**2
#    error = error[~np.isnan(error)]
#    samplenumber = error.size
#    square_error = error.sum()
#    mean_square_error = square_error/samplenumber
#    return mean_square_error
                 
def TypeToFunc(kerneltype):
    dictionary={"gau":gaussian,
                "gaussian":gaussian,
                "qua":quadratic,
                "quadratic":quadratic}
    return dictionary[kerneltype]

def gaussian(dr_matrix):
    answer = np.exp(-3 * dr_matrix)
    #answer[(dr_matrix > 1)] = 0.0
    return answer

def quadratic(dr_matrix):
    #log_manager.info(str(np.sqrt(dr_matrix)))
    answer =  1 - dr_matrix
    answer[dr_matrix > 1] = 0.0
    #log_manager.info(answer)
    #answer[dr_matrix < 0] = 0.0
    return answer

if __name__ == "__main__":
    
    import time
    func=gaussian

    grid_z = np.array([[1,np.nan,3.,4,5],
                          [5,6,1,7,8],
                          [1,np.nan,4,2,5],
                          [5,2,6,3,1.]])
    #grid_trend = grid_z.copy()
    grid_s=np.array([[1,3.],[1,8],[3,2],[4,1]])
    grid_t=np.array([[1,3,5,7,9.]])
    bs=8
    bt=2
    
    print kernelsmoothing(grid_s,grid_t,grid_z,bs,bt,ktype = "gaussian")
    
    print kernelsmoothing_est(grid_s, grid_t, grid_z, 
                        est_grid_s = grid_s, est_grid_t = grid_t,
                        bs = bs, bt = bt, ktype = "gau")


# -*- coding: utf-8 -*-
import numpy as np


def isspacetime(models):
    '''
    analyze nest covariance model expression

    syntax:
        isST, isSTsep, model_res = isspacetime(model)

    input:
        model: a list of covariance model expression

    return:
        isST:    bool, whether model is a space-time model
        isSTsep: bool, whether model is a space-time model 
                       which separate space and time part
        model_res: a list of analyzed covariance model result.
            if isSTsep is True,
                model_res[0] is a list of space covariance model
                model_res[1] is a list of time covariance model
            if isSTsep is False,
                model_res[0] is a list of space|time|space-time
                covarinace model

    note: there is no spelling check with model name
    '''

    isST, isSTsep, model_res =\
        zip(*[parse_model(m) for m in models])

    if not (all(isSTsep) or (not any(isSTsep))):
        raise ValueError(
            'All space-time models must be jointly '
            'either separable or non-separable.')
    if not (all(isST) or (not any(isST))):
        raise ValueError(
            'All models must be jointly either space, time or space-time.')

    return isST[0], isSTsep[0], list(zip(*model_res))

def parse_model(model):
    '''
    analyze single covariance model expression

    syntax:
        isST, isSTsep, model_res = isspacetime(model)

    input:
        model: a covariance model expression

    return:
        isST:    bool, whether model is a space-time model
        isSTsep: bool, whether model is a space-time model 
                       which separate space and time part
        model_res: a list of analyzed covariance model result.
            if isSTsep is True,
                model_res[0] is a space covariance model
                model_res[1] is a time covariance model
            if isSTsep is False,
                model_res[0] is a space|time|space-time
                covarinace model

    note: there is no spelling check with model name
    '''
    if u'/' in model:
        isSTsep = True
        isST = True
        if any(map(lambda x:x.endswith(u'ST'), model.split(u'/'))):
            raise ValueError(
                'Separable model cannot contain another non-separable model.')
    else:
        isSTsep = False
        if model.endswith(u'ST'):
            isST = True
        else:
            isST = False
    if isSTsep:
        return isST, isSTsep, model.split(u'/') # a list
    else:
        # just return not separate model anyway
        return isST, isSTsep, [model]



# -*- coding: utf-8 -*-
import os

from six.moves import range
import numpy as np
from numba.pycc import CC



MODULE_NAME = os.path.split(__file__)[1][:-8] # file name without '__aot.py'

cc = CC(MODULE_NAME)

@cc.export(
    'diffarray', 'Tuple((i8[:],i8[:],f8[:,:],f8[:]))(f8[:,:],f8,b1,b1)'
    )
def diffarray(x, r, include, aniso):
    '''
    Accelerate function "diffarray" with numba

    r == -1.0 means all range to be considered
    '''

    row_count, col_count = x.shape
    d = 0 if include == True else 1

    k=0
    row_count_output = (row_count - d) * (row_count - d + 1) / 2
    index_head = np.empty(row_count_output, dtype = np.int64)
    index_tail = np.empty(row_count_output, dtype = np.int64)
    diff = np.empty((row_count_output, col_count))
    angdiff = np.empty(row_count_output)

    for i in range(row_count - d):
        index_head[k : k+row_count-i-d] = i
        index_tail[k : k+row_count-i-d] = np.arange(row_count)[i+d:]
        diff[k : k+row_count-i-d] = x[i]-x[np.arange(row_count)[i+d:]]
        k = k + row_count - i - d

    if aniso == True:
        angdiff[:] = np.arctan2(diff[:,1], diff[:,0])

    if r != -1.0:
        idxx = np.where(np.sqrt(diff[:,0]**2. + diff[:,1]**2.) <= r)
        index_head = index_head[idxx]
        index_tail = index_tail[idxx]
        diff = diff[idxx]
        if aniso == True:
            angdiff = angdiff[idxx]

    return index_head, index_tail, diff, angdiff

@cc.export(
    'cdist',
    'Tuple((i8[:],i8[:],f8[:,:],f8[:]))(f8[:,:],f8[:,:],f8,b1)'
    )
def cdist(x, y, r, aniso):
    x_row, x_col = x.shape
    y_row, y_col = y.shape
    if x_col != y_col:
        raise ValueError(
            'two array must have the same column length')
    if x_col != 2 and aniso == True:
        raise ValueError(
            'anisotropy is only suit for column length equal to 2')

    index_head = np.empty( x_row * y_row, dtype=np.int64)
    index_tail = np.empty( x_row * y_row, dtype=np.int64)
    for i in range(x_row):
        index_head[i*y_row:(i+1)*y_row] = i
        index_tail[i*y_row:(i+1)*y_row] = np.arange(y_row)

    diff = np.empty((x_row*y_row, x_col))
    for i in range(x_col):
        diff[:,i] = (x[:,i:i+1] - y[:,i]).ravel()
    if aniso == True:
        angdiff = np.arctan2(diff[:,1], diff[:,0]).ravel()
    else:
        angdiff = np.empty(x_row*y_row)

    if r != -1.0:
        dis_arr = np.zeros(x_row*y_row)
        for i in range(x_col):
            dis_arr = dis_arr+diff[:,i]**2

        idxx = np.where(np.sqrt(dis_arr) <= r)
        index_head = index_head[idxx]
        index_tail = index_tail[idxx]
        diff = diff[idxx]
        if aniso == True:
            angdiff = angdiff[idxx]

    return index_head.ravel(), index_tail.ravel(), diff, angdiff.ravel()


if __name__ == "__main__":
    cc.compile()


# -*- coding: utf-8 -*-
import numpy

from .designmatrix import designmatrix


def regression(c, z, order, K):

    X, index = designmatrix(c, order)

    index = index.T
    n, p = X.shape

    if X.size > 0:
        Xt = X.T
        if K.size == 0:
            invXtX = numpy.linalg.inv( Xt.dot(X) )
            best = invXtX.dot(Xt).dot(z)
            zest = X.dot(best)
            resi = z - zest
            s2 = ( resi.T.dot(resi) ) / float(n - p)
            Vbest = invXtX.dot( s2 )
        else:
            XtinvK = X.T.dot( numpy.linalg.inv(K) )
            invXtinvKX = numpy.linalg.inv( XtinvK.dot(X) )
            best = invXtinvKX.dot(XtinvK).dot(z)
            zest = X.dot(best)
            Vbest = invXtinvKX
    else: # set default size
        return numpy.array([],ndmin=2), numpy.array([],ndmin = 2),numpy.array([],ndmin=2), numpy.array([],ndmin = 2)


    return best, Vbest, zest, index



# -*- coding: utf-8 -*-
import csv

import numpy as np
from six.moves import range

from .pystks_variable import get_standard_soft_pdf_type


def ud2ud(file_name, usecols, skiprows, delimiter):
    '''使用者自定型
    mean, var: n by 1 numpy array
    
    '''
    limi_idx = usecols[0] # idx of limi
    f=open(file_name,"rb")
    c=csv.reader(f,delimiter = delimiter )

    #skip rows
    for skiprow in range(skiprows):
        c.next()
    try:
        nol = []
            
        for line in c :
            d=line
            
            a=int(d[limi_idx])+1
            nol.append(a)
                
    except StopIteration:
        pass
    
    nolmax=max(nol)

    f2=open(file_name,"rb")
    readcsv=csv.reader( f2,delimiter= delimiter )
    #skip rows
    for skiprow in range(skiprows):
        readcsv.next()
    try:
        limi = []
        probdens = []
        for i in readcsv:
            d2=i
            a2=int(d2[limi_idx])
            px1=tuple(d2[limi_idx +1 : limi_idx +1 + a2+1])
            px1=px1+tuple([0]*(nolmax-a2 ))
            px=map(float,px1)
            limi.append(px)

            py1=tuple(d2[limi_idx +1 + a2+1 : limi_idx +1 + a2+1+ a2+1])
            py1=py1+tuple([0]*(nolmax-a2 ))
            py=map(float,py1)
            probdens.append(py)
                

    except StopIteration:
        pass
        
        
        
    nl=np.array(nol,ndmin=2).T
    limi=np.array(limi,ndmin=2)
    probdens=np.array(probdens,ndmin=2)
    return nl,limi,probdens

def gs2ud(mean, var):
    '''轉換高斯型資料至使用者自定型
    mean, var: n by 1 numpy array
    
    '''
    softpdftype = 2
    limi_norm = np.array([-3.719,-3.090,-2.326,
                          -1.645,-0.524,-0.253,
                           0.000,
                           0.253, 0.524, 1.645,
                           2.326, 3.090, 3.719,])
    limi_n = len(limi_norm)

    nl = np.ones((mean.shape[0],1), dtype=int) * limi_n
    limi = np.kron(np.sqrt(var),limi_norm).reshape((-1,limi_n))+mean
    probdens = limi-mean
    probdens = -(probdens**2)/2/var
    probdens = 1/np.sqrt(2*np.pi*var)*np.exp(probdens)

    nl, limi, probdens, _ = proba2probdens(softpdftype, nl, limi, probdens)

    return nl, limi, probdens

def uf2ud(low, up):

    softpdftype = 2
    limi_norm = np.linspace(-1,1,5)
    limi_n = len(limi_norm)
    nl = np.ones((low.shape[0],1), dtype=int) * limi_n
    limi = np.kron((up-low)/2.,limi_norm)+(up+low)/2.
    probdens = np.ones((low.shape[0],limi_n))*(1./(up-low))

    nl, limi, probdens, _ = proba2probdens(softpdftype, nl, limi, probdens)

    return nl, limi, probdens

def ud2zs(softpdftype, nl, limi, probdens):
    '''
    user defined to new zs data
    new zs data: a sequence of zsdata,
        zsdata is a sequence of pdftype, *pdf_args
        e.g. zsdata1 = (2, nl, limi, probdens)
        e.g. zsdata2 = (10, mean, var)
        e.g. new zs data = (zsdata1, zsdata2)
    '''
    zsdata = []
    if nl.size != 0:
        for n_i, l_i, p_i in zip(nl, limi, probdens):
            zsdata.append([softpdftype, n_i, l_i, p_i])

    return zsdata

def uf2zs(softpdftype, low, up):
    '''
    new zs data
    '''
    limi_norm = np.linspace(-1,1,5)
    limi_n = len(limi_norm)
    nl = np.ones((low.shape[0],1), dtype=int) * limi_n
    limi = np.kron((up-low)/2.,limi_norm)+(up+low)/2.
    probdens = np.ones((low.shape[0],limi_n))*(1./(up-low))
    zsdata = []
    for n_i, l_i, p_i in zip(nl, limi, probdens):
        zsdata.append([softpdftype, n_i, l_i, p_i])

    return zsdata

def zs2ud(zs):
    '''new zs data'''
    softpdftype = 2
    mean = np.array([zsi[1] for zsi in zs]).reshape((-1,1))
    var = np.array([zsi[2] for zsi in zs]).reshape((-1,1))
    nl, limi, probdens = gs2ud(mean, var)

    return softpdftype, nl, limi, probdens

def proba2probdens(softpdftype, nl, limi, probdens):
    '''
    proba2probdens            - Normalizes the probability density function (Jan 1, 2001)
  
    Calculates the norm (area under the curve) of a function template, and returns
    a normalized pdf. This function uses the syntax of probabilistic data (see 
    probasyntax) to define the pdf.
   
    SYNTAX :
   
    [probdens,norm]=proba2probdens(softpdftype,nl,limi,probdenstemplate);
   
    INPUT :
   
    softpdftype scalar      indicates the type of soft pdf representing the  
                            probabilitic soft data.  
                            softpdftype may take value 1, 2, 3 or 4, as follow:
                            1 for Histogram, 2 for Linear, 3 for Grid histogram, 
                            and 4 for Grid Linear.
                            In current status, only softpdftype 2 is avaiable for use 
    nl          ns by 1     2D array of the number of interval limits. nl(i) is the number  
                            of interval limits used to define the soft pdf for soft data 
                            point i. (see probasyntax for more explanations)
    limi        ns by l     2D array of interval limits, where l is equal to
                            either max(nl) or 3 depending of the softpdftype.
                            limi(i,:) are the limits of intervals for the i-th 
                            soft data. (see probasyntax for more explanations)
    probdenstemplate        ns by p matrix of non-normalized probability density values,  
                            where p is equal to either max(nl)-1 or max(nl), depending on 
                            the softpdftype. probdenstemplate(i,:) are the values of the  
                            non-normalized probability density corresponding to the intervals  
                            for the i-th soft data defined in limi(i,:). (see probasyntax for 
                            more explanations)
   
    OUTPUT :
    nl          ns by 1     2D array of the number of interval limits. nl(i) is the number  
                            of interval limits used to define the soft pdf for soft data 
                            point i. (see probasyntax for more explanations)
    limi        ns by l     2D array of interval limits, where l is equal to
                            either max(nl) or 3 depending of the softpdftype.
                            limi(i,:) are the limits of intervals for the i-th 
                            soft data. (see probasyntax for more explanations)
    probdens    ns by p     2D array of normalized probability density values,
                            each row of probadens is equal to the corresponding row
                            of probadenstemplate divided by its normalization constant
    norm        ns by 1     2D array of the normalization constants (area under the curve)
                            of each row of probadenstemplat, e.g. norm(is) is the
                            normalization constant for probdenstemplate(is,:)  
    ''' 
    softpdftype = 2 # always 2 for now
    
    if type(nl) is int: # only one soft data
      limi = limi.reshape(1, limi.size)
      probdens = probdens.reshape(1, probdens.size)
      nl = np.array(nl).reshape(1, 1)
    elif limi.shape is not probdens.shape:
      limi = limi.reshape((len(nl), int(limi.size/len(nl))))
      probdens = probdens.reshape((len(nl), int(limi.size/len(nl))))
      nl = nl.reshape((len(nl), 1))
    norm_probdens = np.zeros(probdens.shape)
    area = np.zeros(nl.shape)
    m = 0
    for nl_i, limi_i, probdens_i in zip(nl, limi, probdens):
        nl_i = int(nl_i[0])
        limi_i = limi_i[: nl_i]
        probdens_i_original = np.copy(probdens_i) #copy
        probdens_i = probdens_i[:nl_i]

        height = limi_i[1:] - limi_i[:-1]
        sum_up_low = probdens_i[:-1] + probdens_i[1:]
        area[m] = (sum_up_low * height / 2.).sum()
        norm_probdens_i = probdens_i / area[m]
        probdens_i_original[ :nl_i] = norm_probdens_i
        norm_probdens[m]=probdens_i_original#.append( probdens_i_original )
        m = m+1
    
    if len(nl) == 1: 
      limi = limi.reshape(limi.size,)
      norm_probdens = norm_probdens.reshape(limi.size,)    
      
    return nl, limi, norm_probdens, area

def proba2stat(softpdftype, nl, limi, probdens):
    def range_include_end(start, end, step):
        include_end_indexs = np.where( (end - start)%step == 0 )[0]
        revised_end = end.copy()
        revised_end[ include_end_indexs ] += step[ include_end_indexs ]
        result = map( lambda x:np.arange(*x), zip(start, revised_end, step) )
        return result

    softpdftype = get_standard_soft_pdf_type(softpdftype)
    if nl.shape[0] == 0:
        return np.array([]).reshape( (0,1) ), np.array([]).reshape( (0,1) )

    if softpdftype == 1 or softpdftype == 2:
        L1 = limi[:,:-1]
        L2 = limi[:,1:]
    elif softpdftype == 3 or softpdftype == 4: #grid
        nlMax = nl.max()
        limi_expand = np.zeros( (nl.shape[0], nlMax) )
        for idx, i in enumerate( range_include_end(limi[:,0], limi[:,2], limi[:,1]) ):
            limi_expand[idx][:i.size] = i
        L1 = limi_expand[:,:-1]
        L2 = limi_expand[:,1:]

    if softpdftype == 1 or softpdftype == 3:
        P1 = probdens
        XsMean_mat = (1/2.) * P1 * (L2**2 - L1**2)
        Xs2Mean_mat = (1/3.) * P1 * (L2**3 - L1**3)
    elif softpdftype == 2 or softpdftype == 4:
        P1 = probdens[:,:-1]
        P2 = probdens[:,1:]
        fsp = (P2 - P1) / (L2 - L1)
        fso = P1 - L1 * fsp

        L1p2 = L1 * L1
        L1p3 = L1p2 * L1
        L1p4 = L1p3 * L1

        L2p2 = L2 * L2
        L2p3 = L2p2 * L2
        L2p4 = L2p3 * L2
        
        XsMean_mat = ( 1 / 2. ) * ( fso * (L2p2 - L1p2) ) + ( 1 / 3. ) * (fsp * (L2p3 - L1p3) )
        Xs2Mean_mat = ( 1 / 3. ) * ( fso * (L2p3 - L1p3) ) + ( 1 / 4. ) * ( fsp * (L2p4 - L1p4) )

    XsMean = []
    Xs2Mean = []
    for nl_i, XsMean_i, Xs2Mean_i in zip( nl, XsMean_mat, Xs2Mean_mat ):
        XsMean.append( [ XsMean_i[ : nl_i[0] - 1].sum()] )
        Xs2Mean.append( [ Xs2Mean_i[ : nl_i[0] - 1].sum()] )

    XsMean, Xs2Mean = np.array( XsMean ), np.array( Xs2Mean )
    softmean = XsMean
    softvar = Xs2Mean - XsMean**2

    return softmean, softvar

def proba2quantile(softdpftype, nl, limi, probdens, quantiles = []):
    '''give a discrete pdf, return the quantiles user gave'''

    #default quantiles
    if quantiles:
        pass
    else:
        quantiles = [ .05, .25, .50, .75, .95, ]

    
    #get cdf
    probdens_quantile = []
    # probdens_cdf = []

    for nl_i, limi_i, probdens_i in zip( nl, limi, probdens ):

        #clip
        nl_i = int( nl_i[ 0 ] )
        limi_i = limi_i[ : nl_i ]
        probdens_i = probdens_i[ :nl_i ]

        #get area_i
        height = limi_i[ 1: ] - limi_i[ :-1 ]
        sum_up_low = probdens_i[ :-1 ] + probdens_i[ 1: ]
        area_i = (sum_up_low * height / 2.)
        
        #set cdf
        probdens_cdf_i = [ 0. ]
        for p in area_i:
            probdens_cdf_i.append( probdens_cdf_i[ -1 ] + p )

        #get interp
        probdens_quantile.append( np.interp( quantiles, probdens_cdf_i, limi_i ) )

    return np.array( probdens_quantile )

def pdf2cdf(zs):
    Fs = []
    for idx_k, zsi in enumerate(zs):
        softpdftype = zsi[0]
        if softpdftype == 2:
            nl = zsi[1]
            limi = zsi[2]
            probdens = zsi[3]
            height = np.diff(limi)
            sum_up_low = probdens[:-1] + probdens[1:]    
            cumsum = np.hstack([0,np.cumsum(height*sum_up_low/2)])
            probCDFs = cumsum/float(cumsum[nl-1])
            Fs.append([softpdftype,nl,limi,probdens,probCDFs])     
    return Fs

def probaUniform(zlow, zup):
    '''
    function [softpdftype,nl,limi,probdens]=probaUniform(zlow,zup)
    % probaUniform      - creates Uniform soft probabilistic data  at a set of data points
    %
    % This program generates uniform probabilistic soft pdf at a set of data points.
    % At each data point the lower and upper bound of the z values are known to be zlow and zup.
    % The soft probrabilistic distribution is given by the uniform pdf as follow
    % pdf(z)=1/(zup-low) for zlow<z<zup, 0 otherwise.
    %
    % SYNTAX :
    %
    % [softpdftype,nl,limi,probdens]=probaUniform(zlow,zup);
    %
    % INPUT :
    %
    % zlow     ns by 1       vector of the lower bound value at ns soft data point
    % zup      ns by 1       vector of the upper bound value at ns soft data point
    %
    % OUTPUT :
    %
    % softpdftype scalar=2   indicates the type of soft pdf representing the probabilitic soft data.  
    % nl          ns by 1    vector of number of limits  nl (see probasyntax)
    % limi        ns by 2    matrix representing the limits  values (see probasyntax)
    % probdens    ns by 2    matrix representing the probability densities (see probasyntax)
    %
    % SEE ALSO probaGaussian, probaStundentT and probasyntax.m 
    '''

    ns=len(zlow)
    softpdftype=2#*np.ones((ns,1))
    nl=2*np.ones((ns,1)).astype(int)
    zlow=np.array(zlow).reshape((ns,1))
    zup=np.array(zup).reshape((ns,1))
    limi=np.hstack([zlow, zup])
    probdens=np.kron(1./np.diff(limi),[1,1])
    zs=[softpdftype,nl,limi,probdens]
    #zs=[[softpdftype[m],nl[m],limi[m],probdens[m]] for m in range(ns)]

    return zs



import pandas as pd
try:
    import rpy2.robjects as ro
    from rpy2.robjects.packages import importr
    from rpy2.robjects import pandas2ri
except Exception as e:
    print ('Cannot import modeul "rpy2.robjects", try to install "rpy2" first.')
    raise e

rmgcv = importr('mgcv')

from ..stats.dlnm import gam as dlnm_gam


def construct_gam_formula(y=None, x=None, s_x=None, f_x=None, by_str=None):
    '''
    construct r's gam formula

    y: str, response column name
    x: a list of linear function column name
    s_x: a list of smoothing function column name
    f_x: a list of factirial function column name
    by_str: if exists, ignore all variable, create formula by by_str

    return r's gam formula
    '''
    if by_str:
        fmla = ro.Formula(by_str)
    else:
        x_list = []

        if x:
            x_list += x
        if s_x:
            x_list += ['s({i})'.format(i=i) for i in s_x]
        if f_x:
            x_list += ['factor({i})'.format(i=i) for i in f_x]

        x_valid = [i for i in x_list if i]
        if len(x_valid) > 0:
            fmla_str = '{y} ~ '.format(y=y) + ' + '.join(x_valid)
        else:
            raise ValueError('No valid x found.')

        # if not x:
        #     pass
        # else:
        #     pass
        # if not s_x:
        #     s_str = ''
        # else:
        #     s_str = ' + '.join(['s('+i+', k=6)' for i in s_x])
        # if not f_x:
        #     f_str = ''
        # else:
        #     f_str = ' + '.join(['factor('+i+')' for i in f_x])

        # if s_str and f_str:
        #     fmla_str = '{y} ~ {s} + {f}'.format(y=y, s=s_str, f=f_str)
        # elif s_str:
        #     fmla_str = '{y} ~ {s}'.format(y=y, s=s_str)
        # else:
        #     fmla_str = '{y} ~ {f}'.format(y=y, f=f_str)
        fmla = ro.Formula(fmla_str)
    return fmla

def construct_gam_model(gam_formula, data, drop_intercept=None, control=None):
    '''
    gam_formula: r's gam formula (from construct_gam_formula)
    data: pandas dataframe, columns name should fit gam_formula
    control: None or a python dict where it's key is string, (for r list

    return r's gam model
    '''

    if isinstance(control, dict):
        ctl_str = "list("
        kvpair = []
        for k in control:
            if isinstance(control[k], bool):
                kvpair.append('='.join([k, str(control[k]).upper()]))
            else:
                kvpair.append('='.join([k, str(control[k])]))
        ctl_str += ','.join(kvpair)
        ctl_str += ')'
        control = ro.r(ctl_str)


    if drop_intercept:
        model = dlnm_gam(gam_formula, data=data,
            drop_intercept=drop_intercept, control=control)
    else:
        model = dlnm_gam(gam_formula, data=data, control=control)
    return model

def predict_gam(gam_model, data):
    '''
    gam_model: r's gam model (from construct_gam_model)
    data: pandas dataframe, columns name should fit gam_formula

    return gam predict result
    '''
    def _predict_gam(
        obj, newdata, ptype = "link", se_fit = False, terms = ro.NULL,
        block_size = ro.NULL, newdata_gurrantedd = ro.NULL,
        na_action = ro.r('na.pass'), cluster = ro.NULL):
    
        result = rmgcv.predict_gam(obj, newdata, type=ptype, se_fit=se_fit,
                                   terms=terms, block_size=block_size,
                                   newdata_gurrantedd=newdata_gurrantedd,
                                   na_action=na_action, cluster=cluster)
        return result
    value = _predict_gam(gam_model, data)
    return pandas2ri.ri2py(value)

def gam(df, s_x, f_x, y, est_df=None, est_s_x=None, est_f_x=None, est_y=None):
    '''use gam pythonically

    input:
    df: pandas dataframe
    s_x: sequences of non-linear relation column name
    f_x: sequences of factirial relation column name
    y: str, response column name
    est_*: like above, for prediction

    note:
        if est_df is None, gam will fit df data

    return:
    est_df*: est_df with addtional column y(named df's y's column name)
    '''

    r_gam_formula = construct_gam_formula(y, s_x=s_x, f_x=f_x)
    r_gam_model = construct_gam_model(r_gam_formula, data=df)

    if not est_y:
        est_y = y

    if est_df is None:
        est_df = df.copy()
        while est_y in est_df.columns:
            est_y += '_'
        est_df[est_y] = pandas2ri.ri2py(r_gam_model.rx2('fitted.values'))
    else:
        if est_s_x:
            est_df = est_df.rename(
                columns=dict(i for i in zip(est_s_x, s_x)))
        else:
            est_s_x = s_x
        if est_f_x:
            est_df = est_df.rename(
                columns=dict(i for i in zip(est_f_x, f_x)))
        else:
            est_f_x = f_x
        while est_y in est_df.columns:
            est_y += '_' 
        est_df[est_y] = predict_gam(r_gam_model, data=est_df)
        if est_s_x != s_x:
            est_df = est_df.rename(
                columns=dict(i for i in zip(s_x, est_s_x)))
        if est_f_x != f_x:
            est_df = est_df.rename(
                columns=dict(i for i in zip(f_x, est_f_x)))
    return est_df



# -*- coding:utf-8 -*-
import numpy
from scipy.spatial import cKDTree
try:
  from scipy.spatial.distance import cdist as coord2dist
except:
  from .coord2dist import coord2dist  


def neighbours_index_kd(ck, ctree, nmax, dmax):
    '''let ck group by ctree
        ctree can be a numpy array with shape (n, nd) or
        scipy kd-tree

        nmax is first nearest nmax neighbors
        dmax here is only max distance

        return a dict.
            it's key is a tuple, ctree index
            it's value is a list, ck_index
    '''
    if not isinstance(ctree, cKDTree):
        try:
            ctree = cKDTree(ctree)
        except Exception as e:
            import ipdb
            ipdb.set_trace()
    dd, ii = ctree.query(ck, k=range(1, nmax+1), distance_upper_bound=dmax)
    marr = numpy.ma.masked_array(ii, numpy.isinf(dd))
    marr = numpy.sort(marr)

    res = {}
    for k_idx, marr_i in enumerate(marr):
        try:
            res[tuple(marr_i.compressed())].append(k_idx)
        except KeyError:
            res[tuple(marr_i.compressed())] = [k_idx]
    return res
        
def neighbours_kd(c_one, c, z, nmax, dmax, tree=None):
  '''
  input
  c_one: 1 by nd
        nd can be:
            1 for space or time,
            2 for space,
            3 for space-time
  c: n by nd
  z: n by ??
  nmax: int
  dmax: 1 by rd float 
        rd can be:
            1 for space or time
            3 for space-time
  return
  c_nebr, z_nebr, d_nebr, n_nebr, idx_nebr
  '''  
  empty_result = [ numpy.array([]).reshape( ( 0, c_one.shape[1] ) ),
                   numpy.array([]).reshape( ( 0, 1 ) ),
                   numpy.array([]).reshape( ( 0, 1 ) ),
                   0,
                   numpy.array([]).reshape( ( 0, 1 ) ) ]

  isST = 1 if dmax.size == 3 else 0

  if c.size == 0:
    print('no data')
    return empty_result

  if nmax == 0:
    print('nmax is 0')
    return empty_result  
    
  if isST==0: # pure spatial or temporal cases
    if tree is None:
      try:
        tree=cKDTree(c,leafsize=15)
      except:
        import sys
        sys.setrecursionlimit(10000)
        tree=cKDTree(c,leafsize=30)
    d_nebr,idx_nebr=tree.query(
        c_one, k=range(1, nmax+1), distance_upper_bound=dmax[0][0])
    idx_nebr=idx_nebr[0]    
    c_nebr=c[idx_nebr,:]
    z_nebr=z[idx_nebr,:]    
    n_nebr=idx_nebr.size
    return c_nebr, z_nebr, d_nebr, n_nebr, idx_nebr
  
  elif isST == 1:#space time case  
    if tree is None:
      #get distance of time
      d_t = numpy.abs( c[:,2:3] - c_one[:,2:3] )
      index_t = numpy.where( d_t <= dmax[0][1] )
      if len(index_t[0]) == 0:
        print("noneighbor")
        return empty_result
  
      #get distance of space which already match time
      d_xy = coord2dist( c[index_t[0],0:2], c_one[:,0:2] )
      index_s = numpy.where( d_xy <= dmax[0][0] )
      if len(index_s[0]) == 0:
        print("noneighbor")
        return empty_result  
    
      #calculate all distance which matched perfectly
      c_one_n=c_one
      c_n=c[index_t[0][index_s[0]],:]
      c_one_n[:,2]=c_one[:,2]*dmax[0][2]
      c_n[:,2]=c_n[:,2]*dmax[0][2]
      tree=cKDTree(c_n,leafsize=15)

    d_nebr,idx_nebr=tree.query(c_one_n, k=range(1, nmax+1))  
    idx_nebr=index_t[0][index_s[0]][idx_nebr[0]]
    d_nebr=d_nebr.T
    c_nebr=c[idx_nebr,:]
    z_nebr=z[idx_nebr]
    n_nebr=idx_nebr.size   
    return c_nebr, z_nebr, d_nebr, n_nebr,\
        numpy.sort(idx_nebr.reshape((-1,1)), axis=0)
    
    
def neighbours( c_one, c, z, nmax, dmax ):
    '''
    input
    c_one: 1 by nd
        nd can be:
            1 for space or time,
            2 for space,
            3 for space-time
    c: n by nd
    z: n by ??(any), means abservations
    nmax: int
    dmax: 1 by rd float 
        rd can be:
            1 for space or time
            3 for space-time
    return
    c_nebr, z_nebr, d_nebr, n_nebr, idx_nebr
    '''

    empty_result = [ numpy.array([]).reshape( ( 0, c_one.shape[1] ) ),
                     numpy.array([]).reshape( ( 0, 1 ) ),
                     numpy.array([]).reshape( ( 0, 1 ) ),
                     0,
                     numpy.array([]).reshape( ( 0, 1 ) ) ]

    isST = 1 if dmax.size == 3 else 0

    if c.size == 0:
    #    print('no data')
        return empty_result

    if nmax == 0:
    #    print('nmax is 0')
        return empty_result

    if isST == 0: #
        #get distance of space (only)
        d_xy = coord2dist( c, c_one )
        index_s = numpy.where( d_xy <= dmax[0][0] )
        if len(index_s[0]) == 0:
            print("noneighbor")
            return empty_result
        elif len( index_s[0] ) <= nmax:
            c_nebr = c[index_s[0],:]
            z_nebr = z[index_s[0],:]
            d_nebr = d_xy[index_s[0],:]
            n_nebr = len( index_s[0] )
            idx_nebr = index_s[0].reshape( ( -1, 1 ) )
            return c_nebr, z_nebr, d_nebr, n_nebr, idx_nebr
        elif len( index_s[0] ) > nmax:
            d_nebr = d_xy[index_s[0],:]
            index_s = ( numpy.sort( d_nebr[:,0].argsort()[:nmax] ), 0 ) #dummy 0 for consistence
            c_nebr = c[index_s[0],:]
            z_nebr = z[index_s[0],:]
            d_nebr = d_xy[index_s[0],:]
            n_nebr = len( index_s[0] )
            idx_nebr = index_s[0].reshape( ( -1, 1 ) )
            return c_nebr, z_nebr, d_nebr, n_nebr, idx_nebr

    elif isST == 1:#space time case

        #get distance of time
        d_t = numpy.float64(numpy.abs( c[:,2:3] - c_one[:,2:3] ))
        index_t = numpy.where( d_t <= dmax[0][1] )
        if len(index_t[0]) == 0:
            # print("noneighbor")
            return empty_result

        #get distance of space which already match time
        d_xy = coord2dist( c[index_t[0],0:2], c_one[:,0:2] )
        index_s = numpy.where( d_xy <= dmax[0][0] )
        if len(index_s[0]) == 0:
            # print("noneighbor")
            return empty_result
        
        #calculate all distance which matched perfectly
        d_r = d_xy[index_s[0],0:1] + dmax[0][2] * d_t[ index_t[0] [ index_s[0] ],0:1]
        index_r = numpy.where( d_r <= dmax[0][0] + dmax[0][2] * dmax[0][1] )
        
        if len( index_r[0] ) == 0:
            n_nebr = 0
            return empty_result
        elif len( index_r[0] ) <= nmax:
            c_nebr = c[index_t[0],:][index_s[0],:][index_r[0],:]
            z_nebr = z[index_t[0],:][index_s[0],:][index_r[0],:]
            d_nebr = d_r[index_r[0],:]
            n_nebr = len( index_r[0] )
            idx_nebr = index_t[0].reshape( ( -1, 1 ) )[index_s[0][index_r[0]],:]
            return c_nebr, z_nebr, d_nebr, n_nebr, idx_nebr
        elif len( index_r[0] ) > nmax:
            d_nebr = d_r[index_r[0],:]
            index_r = ( numpy.sort( d_nebr[:,0].argsort()[:nmax] ), 0 ) #dummy 0 for consistence
            c_nebr = c[index_t[0],:][index_s[0],:][index_r[0],:]
            z_nebr = z[index_t[0],:][index_s[0],:][index_r[0],:]
            d_nebr = d_r[index_r[0],:]
            n_nebr = len( index_r[0] )
            idx_nebr = index_t[0].reshape( ( -1, 1 ) )[index_s[0][index_r[0]],:]
            return c_nebr, z_nebr, d_nebr, n_nebr, idx_nebr
   



# -*- coding: utf-8 -*-
"""
Created on Tue May 31 11:11:51 2016

@author: hdragon689
"""
import numpy as np
import itertools

def entropyD(*X):
  ''' Empirical entropy estimator for multivariate data
  
  Input 
  
  X   N by d  discrete data with dimension d and observations N
  
  Output
  
  H   scalar  entropy

  http://blog.biolab.si/2012/06/15/computing-joint-entropy-in-python/
  '''  
  
  n_instances = len(X[0])
  H = 0
  for classes in itertools.product(*[set(x) for x in X]):
    v = np.array([True] * n_instances)
    for predictions, c in zip(X, classes):
      v = np.logical_and(v, predictions == c)
    p = np.mean(v)
    H += -p * np.log(p) if p > 0 else 0
  return H
  
def entropyC(data,bins=None):
  '''
  Histogram-based entropy estimation for continuous data
  
  Input:
  data  N by d    data with d dimension and N observations
  bins  sequence or int, optional
      The bin specification:

    * A sequence of arrays describing the bin edges along each dimension.
    * The number of bins for each dimension (nx, ny, ... =bins)
    * The number of bins for all dimensions (nx=ny=...=bins).
    Default is 10 for all dimensions
    
  Output:
  H     scalar    entropy
  
  
  '''
  if bins is None:
    h,edges=np.histogramdd(data,normed=True)
  else:
    h,edges=np.histogramdd(data,normed=True,bins=bins)
  nd=len(edges)
  p=h
  for k in xrange(nd):
    p=p*(np.diff(edges)[k])
  
  idx=np.where(p>0)  
  hh=-p[idx]*np.log(p[idx])
  H=hh.sum()
  return H,p,edges
  
def condentropy(Y,X,bins=None):
  '''
  Estimate conditional entropy and transformation 
  
  condtional entropy
  H(Y|X)=H(X,Y)-H(X)  

  transinformation (mutual information)
  T(X,Y)=H(Y)-H(Y|X)
  
  Input:
  Y     N by 1     2-D array of the conditional entropy to be assessed
  X     N by d     2-D array of the data to be conditioned on
  
  Output:
  cH    scalar     The conditional entropy of Y given X
  tH    scalar     The transinformation T(X,Y), i.e., mutual information I(X,Y)
  H     scalar     The joint entropy of Y and X
  '''
  
  Y=np.array(Y,ndmin=2)
  m,n=Y.shape  
  if m<n:
    Y=Y.T
    m,n=Y.shape

  X=np.array(X,ndmin=2)  
  if X.shape[0] != m:
    X=X.T
    
  A=np.hstack(tuple([Y,X]))
  if bins is None:
    bins=20
  Hx=entropyC(X,bins=bins)[0]
  Hy=entropyC(Y,bins=bins)[0]
  H=entropyC(A,bins=bins)[0]
  cH=H-Hx
  tH=Hy-cH
  return cH,tH,H   

#def entropy(*X):
#    return np.sum(-p * np.log2(p) if p > 0 else 0 
#      for p in (np.mean(reduce(np.logical_and, (predictions == c for predictions, c in zip(X, classes))))
#      for classes in itertools.product(*[set(x) for x in X])))

if __name__ == "__main__":  
   
  '''Test under Multiple Gaussian distribution'''  
  m=[0,0,0]
  cov=[[1,0.3,0.6],[0.3,1,0.5],[0.6,0.5,1]]
  data2=np.random.multivariate_normal(m,cov,10000)
  
  Hn,pn,edges=entropyC(data2,bins=(1000,1000,1000))
  dx=np.diff(edges)[:,0]
  
  HnT=3./2*np.log(2*np.pi)+0.5*np.log(np.linalg.det(cov))+3./2-np.log(np.prod(dx))  
  
  print Hn
  print HnT

 


  


# -*- coding: utf-8 -*-
"""
Created on Fri Aug  7 01:23:34 2015

@author: hdragon689
"""

from six.moves import range
import numpy


def gs(X, row_vecs=True, norm = True):
  '''
  Gram-Schmidt orthogonalization

  Syntax: Y = gs(X, row_vecs=True, norm = True)
  
  Input:
  X           m by n      a 2D array with data to be orthogonalized
  row_vecs    bool        True if X is row-wise; otherwise, it is column-wise
                          default is True    
  norm        bool        
    
  Remark:
  Row-wise data implies X=[[g1],[g2],[g3],...] where gi vector has shape of 1 by n 
  Column-wise data implies X=[[g1,g2,g3,]] where gi vector has shape of n by 1    
  
  The original code is downloaded from https://gist.github.com/iizukak/1287876    
  '''  
  if not row_vecs:
    X = X.T
  Y = X[0:1,:].copy()
  for i in range(1, X.shape[0]):
    proj = numpy.diag((X[i,:].dot(Y.T)/numpy.linalg.norm(Y,axis=1)**2).flat).dot(Y)
  #  print(proj)
    Y = numpy.vstack((Y, X[i,:] - proj.sum(0)))
  if norm:
    Y = numpy.diag(1/numpy.linalg.norm(Y,axis=1)).dot(Y)
  if row_vecs:
    return Y
  else:
    return Y.T    

def wqr(X, col_vecs=True, norm = True, wt=None):
  '''
  Weighted QR decomposition by Gram-Schmidt process
  
  Syntax: Q,R = wqr(X, col_vecs=True, norm = True, wt=None)
  
  Input:
  X           m by n      a 2D array with data to be orthogonalized
  col_vecs    bool        True if X is column-wise; otherwise, it is row-wise
                          default is True (column-wise)    
  norm        bool        True to provide the normalized orhtogonal vectors
  wt          m by 1      a 2D array with weighting function for orthogonalization process. 
                          Default considers the uniform weights for the vectors  
  
  Return:
  Q           m by n      a 2D array of orthogalized vectors 
  R           n by n      the upper triangular matrix such that X=QR
    
  Remark:
  1) Row-wise data implies X=[[g1],[g2],[g3],...] where gi vector has shape of 1 by n 
     Column-wise data implies X=[[g1,g2,g3,]] where gi vector has shape of n by 1   
  2) If row-wise data is provided, the inversion relationship will be A=R*Q
  

  '''
  if not col_vecs:
    X = X.T

  m,n=X.shape   
  
  if wt is None:
    wt=numpy.ones(X[:,0:1].shape)    
  else:
    wt=wt.reshape(m,1)
  wtdlg=numpy.diag(wt.flat)
    
  R=numpy.zeros((n,n))
  Rt=numpy.zeros((n,n)) 
  Y = X[:,0:1].copy()
  for i in range(1, n):
    Rt[i,0:i]=numpy.dot(Y.T.dot(wtdlg),X[:,i:i+1]).T
    proj= (numpy.diag((Rt[i,0:i]/numpy.dot(wt.T,Y*Y)).flat).dot(Y.T)).T
    Rt[i,0:i]=Rt[i,0:i]/numpy.sqrt(numpy.dot(wt.T,Y*Y))# /numpy.sqrt(numpy.dot(wt.T,Y[:,i-1:i]*Y[:,i-1:i]))
    Y = numpy.hstack((Y, X[:,i:i+1] - proj.sum(1).reshape(m,1)))
  R=Rt.T
  for i in range(n):
    R[i,:]=R[i,:]/numpy.sqrt(numpy.dot(wt.T,Y[:,i:i+1]*Y[:,i:i+1]))
  # numpy.fill_diagonal(R,numpy.sqrt(numpy.dot(wt.T,Y*Y)).flat)
    numpy.fill_diagonal(R,numpy.ones(n))
  if norm:
    R = numpy.dot(numpy.diag(numpy.sqrt(numpy.dot(wt.T,Y*Y)).flat),R)
    Y = numpy.dot(Y,numpy.diag((numpy.sqrt(1/numpy.dot(wt.T,Y*Y)).flat)))    
  if col_vecs:
    return Y,R
  else:
    return Y.T,R.T 
    
if __name__ == "__main__":
  X=numpy.random.rand(5,10)
  Q1=gs(X)
  Q2,R2=wqr(X.T)
  print(numpy.allclose(Q1.T,Q2))
  print(numpy.allclose(X.T,Q2.dot(R2)))
  I=numpy.diag(numpy.ones(X.shape[0]))
  print(numpy.allclose(I,(Q2.T).dot(Q2)))
    


# -*- coding: utf-8 -*-
import numpy

class BMEoptions(object):
    def __init__(self):
        options = numpy.array([
            [0.],  # 1
            [0.0001],  # 2
            [10000.],  # 3 is the maximum number of evaluation of the integral
            [0.001],  # 4 is the relative error on the estimation of the integral
            [0.],  # 5 (test)
            [25.],  # 6
            [0.001],  # 7
            [2.],  # 8 number of moments calculated
                  # 1 for mean, 2 for mean and std dev, 3 for all three moments)
            [0.],  # 9 (test)
            [0.],  # 10 (test)
            [0.],  # 11 (test)
            [0.],  # 12 (test)
            [0.],  # 13 (test)
            [100.],  # 14
            [0.],  # 15 (test)
            [0.],  # 16 (test)
            [0.],  # 17 (test)
            [0.],  # 18 (test)
            [0.],  # 19 (test)
            [0.68],  # 20
            [0.],  # 21
            [0.],  # 22
            [0.],  # 23
            [0.],  # 24
            [0.],  # 25
            [0.],  # 26
            [0.],  # 27
            [0.],  # 28
            [0.],  # 29
            ])

        #make options pythonic
        self.options_dict = {}
        for idx, k in enumerate(options):
            self.options_dict[idx] = k
            self.options_dict[idx,0] = k[0]

        #we can add more options with key and value here
        #and old code still compatible
        self.options_dict['integration method'] = 'qmc'
        self.options_dict['qmc_showinfo'] = False
        self.options_dict['ck pdf debug'] = False
        self.options_dict['debug'] = False

    def __setitem__(self, key, value):
        if type(key) == tuple:
            self.options_dict[key] = value
            self.options_dict[key[0]] = [value]
        elif type(key) == str:
            self.options_dict[key] = value
        else:
            raise TypeError(
                'Not a valid key type: {t}'.format(t=type(key))
                )

    def __getitem__ (self, key):
        return self.options_dict[key]



# -*- coding: utf-8 -*-
"""
Created on Sun Oct  4 23:53:36 2015

@author: hdragon689
"""
import matplotlib.pyplot as plt

from ..bme.softconverter import proba2probdens

def softpdfplot(limi, proba, softpdftype=2, nl=None):
    '''
    argument format can refer to the function of
    stamps.bme.softconverter.proba2probdens
    '''
    if nl is None:
      nl = len(limi)    
    
    probdens = proba2probdens(softpdftype, nl, limi, proba)  
    plt.plot(limi, probdens)
    plt.show()
  
def mepdfplot(mepdf, npar, dim):
    '''
    Input:
    mepdf    func
    '''
